##########################################################################################################
## Program: analysisDescr
## Description: generates unformatted descriptive tables
##########################################################################################################

print("Generating descriptive analysis output")

##########################################################################################################
## Load Clean Data

if (!"dt.time.varying" %in% ls()) {

  dt.time.varying <- readRDS(paste0(cleanDir, "/dt_time_varying.Rds"))

}

if (!"dt.time.invariant" %in% ls()) {

  dt.time.invariant <- readRDS(paste0(cleanDir,"/dt_time_invariant.Rds"))

}

if (!"dt.applications.raw" %in% ls()) {

  dt.applications.raw <- readRDS(paste0(cleanDir, "/dt_syep_applications.Rds"))

}

##########################################################################################################
## Table 1: Sample by Lottery Outcome, 2005 - 2008

# compute lottery winners and losers by year
tab.1.l <- dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0, .(num.students = .N), by = .(select.prev, year)]

tab.1 <- dcast(tab.1.l, year ~ select.prev, value.var = "num.students")
setnames(tab.1, c("year", "0", "1"), c("year", "lottery.losers", "lottery.winners"))
tab.1[, total := lottery.winners + lottery.losers]
tab.1[, syepyear := year -1 ]

# add column-wise totals
tab.1 <- rbind(tab.1, tab.1[, .(lottery.winners = sum(lottery.winners), lottery.losers = sum(lottery.losers), total = sum(total), syepyear = "Total")], fill = TRUE)

saveRDS(tab.1, file = paste0(tabDir, "table1.Rds"))

##########################################################################################################
## Table 2: Comparison of Applicants and Non-Applicants

# define first-time applicants as those with first application year in 2006-2008 and grades 8-11 and select alt programs
dt.time.varying[, first.app.flag := as.numeric(grade %in% c(8, 9, 10, 11, 96, 97) & first.app.year.incl.drop != 2005 &
  year == first.app.year.incl.drop & !is.na(first.app.year.incl.drop))]

# flag grade of first application
dt.time.varying[, grade.first.app := grade[first.app.flag == 1], by = .(sid)]

# flag comparison group as those in grades 8-11 and select alt program
dt.time.varying[, non.app.comp := as.numeric(year != 2005 & grade != 12 & grade != 0 & grade.drop == 0 & ever.apply.0508.incl.drop == 0 & district.drop == 0 & year != 2009)]

## Panel A: Comparison of Applicants and Non-Applicants

# set covariates to test
covariate.list <- c("female.mode","white.mode","black.mode","hispanic.mode","asian.mode","freelu.mode","redlu.mode", "lep.mode","esl.not.lep.mode","sped.mode","zread8","zmath8")

tab.2.covariate.diffs <- lapply(covariate.list, test.mean.diffs,
  dt.1 = unique(dt.time.varying[first.app.flag == 1, c("sid", covariate.list), with = FALSE]),
  dt.2 = unique(dt.time.varying[non.app.comp == 1, c("sid", covariate.list), with = FALSE]))

# extract columns for table from list of covariate differences
tab.2.mean.1 <- unlist(lapply(tab.2.covariate.diffs, function(x) round(x[[1]],3)))

tab.2.mean.2 <- unlist(lapply(tab.2.covariate.diffs, function(x) round(x[[2]],3)))

tab.2.frac.diff <- unlist(lapply(tab.2.covariate.diffs, function(x) round(x[[3]],3)))

tab.2.p.value <- unlist(lapply(tab.2.covariate.diffs, function(x) round(x[[4]],3)))

tab.2.panel.a <- as.data.table(cbind(covariate.list, tab.2.mean.1, tab.2.mean.2, tab.2.frac.diff, tab.2.p.value))

saveRDS(tab.2.panel.a, file = paste0(tabDir, "table2a.Rds"))

## Panel B: Grade of First Application

# bucket alternative grades
dt.time.varying[grade.first.app %in% c(96, 97), grade.tab.2 := 999]
dt.time.varying[!grade.first.app %in% c(96, 97), grade.tab.2 := grade]

# table of students by grade of first application
tab.2.panel.b <- dt.time.varying[first.app.flag == 1 & app.drop.flag == 0, 
  .(
    n.students = .N,
    frac.students = .N / (nrow(dt.time.varying[first.app.flag == 1 & app.drop.flag == 0]))
  ), 
  by = .(grade.tab.2)]

# add sum totals
count.never.apply <- length(unique(dt.time.varying[non.app.comp == 1, sid]))
count.apply <- length(unique(dt.time.varying[first.app.flag == 1 & !is.na(grade.tab.2) & app.drop.flag == 0, sid]))

row.sum.apply <- c("Total Apply", count.apply)
row.sum.never.apply <- c("Total Never Apply", count.never.apply)

row.sums <- data.table(rbind(row.sum.apply, row.sum.never.apply))
colnames(row.sums) <- c("grade.tab.2", "n.students")

tab.2.panel.b.counts <- rbind(tab.2.panel.b[order(grade.tab.2)], row.sums, fill = TRUE)

tab.2.panel.b.counts[grade.tab.2 == 999, grade.tab.2 := "Alt Specialized Program"]

saveRDS(tab.2.panel.b.counts, file = paste0(tabDir, "table2b.Rds"))

##########################################################################################################
## Table 3 - Regents Exam Outcomes in Year Following SYEP (Applicants 2005 - 2008)

tab.3.l <- dt.time.varying[year %in% seq(2006, 2009) & apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0, 
 .(
   var.names = names(.SD),
   count = lapply(.SD, function(x) length(sid[!is.na(x)])),
   mean = lapply(.SD, mean, na.rm = TRUE),
   sd = lapply(.SD, function(x) var(x, na.rm = TRUE)^0.5),
   min = lapply(.SD, min, na.rm = TRUE),
   max = lapply(.SD, max, na.rm = TRUE)
 ), .SDcols = c("ever_attempt", "n_attmpt", "evr_pass65", "n_pass55", "n_pass65", "n_pass75", "ave_score2zyr")]

saveRDS(tab.3.l, file = paste0(tabDir, "table3.Rds"))

##########################################################################################################
## Table 4 - SYEP Take-Up (2005 - 2008)

# fraction of winners that worked and count of winners by year
tab.4.by.year <- dt.time.varying[year %in% seq(2006, 2009) & select.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0,
  .(
    frac.worked = mean(worked.prev)*100, 
    number.winners = sum(select.prev)
    ), by = .(year)]

setkey(tab.4.by.year, year)

# retard year to be application year
tab.4.by.year[, syepyear := year - 1]

# totals
tab.4.totals <- dt.time.varying[year %in% seq(2006, 2009) & select.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0,
  .(
    syepyear = "Total", 
    frac.worked = mean(worked.prev)*100, 
    number.winners = sum(select.prev))]

# stack total
tab.4 <- rbind(tab.4.by.year, tab.4.totals, fill = TRUE)

saveRDS(tab.4, file = paste0(tabDir, "table4.Rds"))

##########################################################################################################
## Table 9: Number of Applications and Selections

## Applications

# fraction of sample by number of applications
count.frac.by.num.app <- dt.time.invariant[num.app > 0, 
  .(frac.app = .N / nrow(dt.time.invariant[num.app > 0])*100, 
    count = .N), 
  by = .(num.app)]

# add total
app.totals <- data.table(cbind(100, sum(count.frac.by.num.app$frac.app), sum(count.frac.by.num.app$count)))
colnames(app.totals) <- c("num.app", "frac.app", "count")

tab.9.app <- rbind(count.frac.by.num.app, app.totals)

## Wins

# fraction of sample by number of applications
count.frac.by.wins <- dt.time.invariant[num.app > 0, 
  .(frac.win = .N / nrow(dt.time.invariant[num.app > 0])*100, 
    count.win = .N), 
  by = .(num.win)]

# add total
win.totals <- data.table(cbind(100, sum(count.frac.by.wins$frac.win), sum(count.frac.by.wins$count.win)))
colnames(win.totals) <- c("num.win", "frac.win", "count.win")

tab.9.win <- rbind(count.frac.by.wins, win.totals)

## Combine

tab.9 <- merge(tab.9.app, tab.9.win, by.x = "num.app", by.y = "num.win", all = TRUE)
tab.9[, num.app := as.character(num.app)]
tab.9[, num.win := as.character(num.app)]
tab.9[num.app == "100", num.app := "Total"]
tab.9[num.win == "100", num.win := "Total"]
tab.9[num.app == "0", num.app := ""]

saveRDS(tab.9, file = paste0(tabDir, "table9.Rds"))

##########################################################################################################
## Table A1: Winning by Match Status

tab.a1 <- dt.applications.raw[app.drop.flag == 0 & syepyear %in% 2005:2008,
  .(win.prob.matched = mean(select[matched == 1])*100,
    win.prob.not.matched = mean(select[matched == 0])*100,
    win.prob.total = mean(select)*100,
    n.app.matched = sum(as.numeric(matched == 1)),
    n.app.not.matched = sum(as.numeric(matched == 0)),
    n.app.total = .N
    ), by=.(syepyear)]

setkey(tab.a1, syepyear)

saveRDS(tab.a1, file = paste0(tabDir, "tableA1.Rds"))

##########################################################################################################
## Table A3: Attrition in Year Following Application

# flag alternative special program
dt.time.varying[!grade.prev %in% c(96,97), grade2.prev := grade.prev]
dt.time.varying[!grade.prev %in% c(96,97), grade2.prev := grade.prev]
dt.time.varying[grade.prev %in% c(96,97), grade2.prev := 100]
dt.time.varying[grade.prev == 0, grade2.prev := 200]

tab.a3 <- dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & year %in% 2006:2009 & !is.na(grade2.prev) & grade2.prev != 200,
  .(frac.attrit.all = mean(1 - in.student.files)*100,
    frac.attrit.win = mean(1 - in.student.files[select.prev == 1])*100,
    frac.attrit.lose = mean(1 - in.student.files[select.prev == 0])*100,
    n.apps = .N
  ), by = .(grade2.prev)]
setkey(tab.a3, grade2.prev)

tab.a3.tot <- dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & year %in% 2006:2009 & !is.na(grade2.prev) & grade2.prev != 200,
  .(frac.attrit.all = mean(1 - in.student.files)*100,
    frac.attrit.win = mean(1 - in.student.files[select.prev == 1])*100,
    frac.attrit.lose = mean(1 - in.student.files[select.prev == 0])*100,
    n.apps = .N
  )]

tab.a3.out <- rbind(tab.a3, tab.a3.tot, fill = TRUE)

tab.a3.out[is.na(grade2.prev), grade.label := "Total"]
tab.a3.out[grade2.prev == 100, grade.label := "Alt. Program"]
tab.a3.out[grade2.prev == 200, grade.label := "Missing Grade"]
tab.a3.out[is.na(grade.label), grade.label := as.character(grade2.prev)]

saveRDS(tab.a3.out, file = paste0(tabDir, "tableA3.Rds"))

##########################################################################################################
## Table A5: Fraction Attempting at Least 1 Exam

# flag alternative special program
dt.time.varying[!grade %in% c(96,97), grade2 := grade]
dt.time.varying[!grade %in% c(96,97), grade2 := grade]
dt.time.varying[grade %in% c(96,97), grade2 := 100]
dt.time.varying[grade == 0, grade2 := 200]

tab.a5 <- dt.time.varying[in.student.files == 1 & year %in% 2006:2009 & !is.na(grade2), .(frac.ever_attempt = mean(ever_attempt)*100), by = .(grade2)]

setkey(tab.a5, grade2)

tab.a5[is.na(grade2), grade.label := "Total"]
tab.a5[grade2 == 100, grade.label := "Alt. Program"]
tab.a5[is.na(grade.label), grade.label := as.character(grade2)]

saveRDS(tab.a5[grade2 %in% c(8,9,10,11,12,100)], file = paste0(tabDir, "tableA5.Rds"))

##########################################################################################################
## Misc In-Text Numbers from Draft

## Applications and Selections in 2005 and 2008 (Note: only cleaning is dropping duplicate applications)

# applications
dt.applications.raw[has.dupe.app == 0 & syepyear %in% c(2005,2008), .N, by = .(syepyear)]

# selections
dt.applications.raw[has.dupe.app == 0 & syepyear %in% c(2005,2008), sum(select), by = .(syepyear)]

## Total Applications in Sample (Pre-Cleaning)

dt.time.varying[year %in% 2005:2008, sum(apply)]

## Match Rate of Applications (Note: this is limiting to clean applications only)

dt.applications.raw[app.drop.flag == 0 & syepyear %in% 2005:2008, mean(matched), by=.(syepyear)]

## Applications in Sample

dt.time.varying[year %in% 2005:2008, sum(as.numeric(apply == 1 & app.drop.flag == 0)), by=.(year)]

## N Unique Applicants

length(unique(dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & year %in% seq(2006,2009), sid]))
  
## N Cohorts

length(unique(dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & year %in% seq(2006,2009), cohort.fe]))

## Examined Appliers Who Repeat a Grade

dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & year %in% seq(2006, 2009) & grade.prev %in% c(8, 9, 10, 11), sum(as.numeric(grade.prev == grade))]

dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & year %in% seq(2006, 2009) & grade.prev %in% c(8, 9, 10, 11), mean(as.numeric(grade.prev == grade))]
