##########################################################################################################
## Program: analysisReg
## Description: generates unformatted regression tables
##########################################################################################################

print("Generating regression and EB analysis output")

##########################################################################################################
## Load Datasets
  
if (!"dt.time.varying" %in% ls()) {

  dt.time.varying <- readRDS(paste0(cleanDir, "/dt_time_varying.Rds"))

}

if (!"dt.time.invariant" %in% ls()) {

  dt.time.invariant <- readRDS(paste0(cleanDir,"/dt_time_invariant.Rds"))

}

if (!"dt.applications.raw" %in% ls()) {

  dt.applications.raw <- readRDS(paste0(cleanDir, "/dt_syep_applications.Rds"))

}

##########################################################################################################
## Global Options and Objects

## Outcomes

outcome.list <- c("ever_attempt","n_attmpt","evr_pass65","n_pass55","n_pass65","n_pass75","ave_score2zyr")
outcome.labels <- c("Any Attempt","N. Attempts","Any Pass 65","N. Pass 55","N. Pass 65","N. Pass 75","ZScore")

# combine outcome list and labels
outcome.list.labels <- list()
for (i in 1:length(outcome.list)) {
  
  outcome.list.labels[[i]] <- c(outcome.list[[i]], outcome.labels[[i]])

}

## RHS Variables

rhs.covariates <- c("female","miss_gender","black","asian","hispanic","miss_ethn","freelu","redlu","missfrl",
  "lep","esl.not.lep","sped","age_syep_rec","age_flag", "zread8_rec","zread8_flag","zmath8_rec","zmath8_flag")

## Treatment and Instrument

treatment.var <- c("worked.prev")
instrument.var <- c("select.prev")

## Fixed Effects and Clustering

fixed.effects.apps <- c("cohort.fe","grade.prev.fe","cbo.prev.year.fe")
fixed.effects.all <- c("year.fe","grade.fe")
fixed.effects.apps.alt <- c("cohort.fe","grade.year.prev.fe","cbo.prev.year.fe")

cluster <- c("sid")

##########################################################################################################
## Table 5 - Lottery Randomization

# test restricted model on multiple covariates by year
mvreg.char.year <- function(dt, LHS.names, RHS.full, RHS.restrict, year.i){
  
  Y.char.year <- as.matrix(dt[(year == year.i), LHS.names, with = FALSE])
  
  model.year <- lm(as.formula(paste0("Y.char.year ~ ", RHS.full)), data = dt[(year == year.i)])
  
  model.year.restricted <- lm(as.formula(paste0("Y.char.year ~ ", RHS.restrict)), data = dt[(year == year.i)])
  
  anova.out <- anova(model.year, model.year.restricted)
  
  return(anova.out)
}

# column names for LHS of multiple regressions matrix of covariates to regress on winning and provider 
covariate.list <- c("female","white","asian","black","hispanic","freelu","redlu","lep","esl","sped",
  "grade8","grade9","grade11","grade97","age_syep_rec","zread8_rec","zmath8_rec")

# RHS for full and restricted models
RHS.full <- "cbo.fe*select - select"
RHS.restrict <- "cbo.fe"

year.list <- seq(2005, 2008)

yearly.anova <- lapply(year.list, mvreg.char.year, dt = dt.time.varying[apply == 1 & app.drop.flag == 0], 
  LHS.names = covariate.list, RHS.full = RHS.full, RHS.restrict = RHS.restrict)

# extract F statistic and p-val
tab.5.F.stat <- unlist(lapply(yearly.anova, function(x) x[[5]][[2]]))

tab.5.p.val <- unlist(lapply(yearly.anova, function(x) x[[8]][[2]]))

tab.5 <- data.table(cbind(c("F", "Prob > F"), data.table(rbind(tab.5.F.stat, tab.5.p.val))))
colnames(tab.5) <- c("", year.list)

saveRDS(tab.5, file = paste0(tabDir, "table5.Rds"))

##########################################################################################################
## Table 6 - OLS ITT Estimates

# OLS on each outcome (ITT model)
itt.ols.list <- lapply(outcome.list, reg.OLS, dt = dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0], 
  regressors = c("select.prev", rhs.covariates), 
  fixed.effects = fixed.effects.apps, cluster.group = cluster)

tab.6 <- stargazer(itt.ols.list)

saveRDS(tab.6, file = paste0(tabDir, "table6.Rds"))

# save p-values for in-text placement
itt.p.vals <- lapply(itt.ols.list, function(x) x$cpval[["select.prev"]])

saveRDS(itt.p.vals, file = paste0(tabDir, "ittPVals.Rds"))

##########################################################################################################
## Table 7 - 2SLS TOT Estimates

# 2SLS on each outcome (TOT model)
tot.2sls.list <- lapply(outcome.list, reg.2SLS, dt = dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0], 
  treatment.var = treatment.var, instrument.var = instrument.var, 
  regressors = rhs.covariates, fixed.effects = fixed.effects.apps, cluster.group = cluster)

tab.7 <- stargazer(tot.2sls.list)

saveRDS(tab.7, file = paste0(tabDir, "table7.Rds"))

# extract R2 from projection model
tab.7.R2 <- sapply(tot.2sls.list, function(x) summary(x)$P.r.squared)

saveRDS(tab.7.R2, paste0(tabDir, "table7_R2.Rds"))

# store LATE coefficients for each outcome
LATE.2sls.list <- lapply(tot.2sls.list, function(x) coef(x)["`worked.prev(fit)`"])

saveRDS(LATE.2sls.list, paste0(tabDir, "totLATE.Rds"))

##########################################################################################################
## LR Test for Tables 6 and Tables 7 (Outcomes 1 - 6)

# likelihood ratio test
LR.test <- function(model.u, model.r, num.restrictions){
  
  Sigma.u <- cov(residuals(model.u))
  Sigma.r <- cov(residuals(model.r))
  
  chi.sq <- (model.u$N)*(log(det(Sigma.r)) - log(det(Sigma.u)))
  
  p.val <- pchisq(chi.sq, df = num.restrictions, lower.tail = FALSE)
  
  lr.out <- as.data.table(c(chi.sq, p.val, num.restrictions))
  lr.out[, variable := c("chi.sq","p.val","restrictions")]
  
  return(lr.out)
}

outcome.matrix <- as.matrix(dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0][, outcome.list[1:6], with = FALSE])

# restricted model
model.r <- felm(as.formula( paste0("outcome.matrix ~ ", paste(rhs.covariates, collapse = "+"), "|", paste(fixed.effects.apps, collapse = "+")) ), 
  data = dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0])

## ITT

table.6.model.u <- felm(as.formula( paste0("outcome.matrix ~ ", "select.prev + ", paste(rhs.covariates, collapse = "+"), "|", paste(fixed.effects.apps, collapse = "+")) ), 
  data = dt.time.varying[apply.prev==1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0])

table.6.lr.test <- LR.test(table.6.model.u, model.r, 6)
setnames(table.6.lr.test, old = c("V1"), new = "ITT")

## TOT

table.7.model.u <- felm(as.formula( paste0("outcome.matrix ~ ", paste(rhs.covariates, collapse = "+"), "|", paste(fixed.effects.apps, collapse = "+"), 
  "| (worked.prev ~ select.prev)") ), 
  data = dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0])

table.7.lr.test <- LR.test(table.7.model.u, model.r, 6)
setnames(table.7.lr.test, old = c("V1"), new = "TOT")

##########################################################################################################
## Table 7 Robustness - Comparison with and Without US History

# 2SLS without US history
tot.2sls.list.no.us <- lapply(paste0(outcome.list, "_noUS"), reg.2SLS, dt = dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0], 
  treatment.var = treatment.var, instrument.var = instrument.var, 
  regressors = rhs.covariates, fixed.effects = fixed.effects.apps, cluster.group = cluster)

tab.7.no.us <- stargazer(tot.2sls.list.no.us)

saveRDS(tab.7.no.us, file = paste0(tabDir, "table7noUS.Rds"))

##########################################################################################################
## Table 10: Heterogeneity by Past Participation

## Specify Models for Table 10

regressors.10 <- rhs.covariates
fixed.effects.10 <- fixed.effects.apps

#= Define Groups: first-time applicants, second-time applicants (one-time past participators), third-time applicants (two-time past participators) =#

dt.time.varying[, het.effects.group.1 := as.numeric(apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & 
  cum.num.apps.incl.drop.prev == 0 & year %in% seq(2007, 2009))]

dt.time.varying[, het.effects.group.2 := as.numeric(apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & 
  cum.num.apps.incl.drop.prev == 1 & cum.num.worked.incl.drop.prev == 1 & year %in% seq(2007, 2009))]

dt.time.varying[, het.effects.group.3 := as.numeric(apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0 & 
  cum.num.apps.incl.drop.prev == 2 & cum.num.worked.incl.drop.prev == 2 & year %in% seq(2007, 2009))]

dt.time.varying[het.effects.group.1 == 1, het.effects.group := 1]
dt.time.varying[het.effects.group.3 == 1, het.effects.group := 3]
dt.time.varying[het.effects.group.2 == 1, het.effects.group := 2]

dt.time.varying[, het.effects.group := as.factor(het.effects.group)]

#= Table 10: Treatment Effect for Each Group =#

# first time 
table.10a.model <- lapply(outcome.list, reg.OLS, dt = dt.time.varying[het.effects.group.1 == 1], 
  regressors = c("select.prev", regressors.10), fixed.effects = fixed.effects.10)

tab10a <- stargazer(table.10a.model)

saveRDS(tab10a, file = paste0(tabDir, "table10a.Rds"))

# second-time applicants, one-time past participators
table.10b.model <- lapply(outcome.list, reg.OLS, dt = dt.time.varying[het.effects.group.2 == 1], 
  regressors = c("select.prev", regressors.10), fixed.effects = fixed.effects.10)

tab10b <- stargazer(table.10b.model)

saveRDS(tab10b, file = paste0(tabDir, "table10b.Rds"))

# third-time applicants, second-time past participators
table.10c.model <- lapply(outcome.list, reg.OLS, dt = dt.time.varying[het.effects.group.3 == 1], 
  regressors = c("select.prev", regressors.10), fixed.effects = fixed.effects.10)

tab10c <- stargazer(table.10c.model)

saveRDS(tab10c, file = paste0(tabDir, "table10c.Rds"))

##########################################################################################################
## Table 11: Balance Test - Heterogeneity by Past Participation

cov.list.to.test <- c("female","white","black","asian","hispanic","freelu","redlu","lep","esl.not.lep","sped","age_syep_rec","zread8","zmath8")
cov.list.miss.flag <- c("miss_gender","miss_ethn","miss_ethn","miss_ethn","miss_ethn","missfrl","missfrl","0","0","0","age_flag","zread8_flag","zmath8_flag")
covariate.names <- c("Female","White","Black","Asian","Hispanic","Free Lunch","Reduced Lunch","LEP","ESL Not LEP","Spec Ed","Age","Zread","Zmath")

balance.test <- function(covariate.name, covariate.miss.flag){
  
  dt.time.varying[, temp.cov := get(covariate.name)]
  
  if (covariate.miss.flag != "0"){
    
    dt.time.varying[, temp.cov.miss.flag:=get(covariate.miss.flag)]
    
    t.test.1.2 <- t.test(dt.time.varying[het.effects.group == 1 & temp.cov.miss.flag == 0, temp.cov], dt.time.varying[het.effects.group == 2 & temp.cov.miss.flag == 0, temp.cov])
    t.test.2.3 <- t.test(dt.time.varying[het.effects.group == 2 & temp.cov.miss.flag == 0, temp.cov],dt.time.varying[het.effects.group == 3 & temp.cov.miss.flag == 0, temp.cov])
  
    dt.time.varying[, temp.cov.miss.flag := NULL]
    
  } else {
  
    t.test.1.2 <- t.test(dt.time.varying[het.effects.group == 1, get(covariate.name)], dt.time.varying[het.effects.group == 2, temp.cov])
    t.test.2.3 <- t.test(dt.time.varying[het.effects.group == 2, get(covariate.name)], dt.time.varying[het.effects.group == 3, temp.cov])
    
  }
  
  dt.time.varying[, temp.cov := NULL]
  
  list.out <- list(list(t.test.1.2$estimate[[1]], t.test.1.2$estimate[[2]], t.test.2.3$estimate[[2]]), list(t.test.1.2$p.val, t.test.2.3$p.val))
  
  return(list.out)
}

balance.test.all.cov <- mapply(balance.test, covariate.name = cov.list.to.test, covariate.miss.flag = cov.list.miss.flag, SIMPLIFY = FALSE)

# build table
group.means <- lapply(balance.test.all.cov, function(x) unlist(x[[1]]))
p.vals <- lapply(balance.test.all.cov, function(x) unlist(x[[2]]))

tab.11 <- as.data.table(
  cbind(
    covariate.names, 
    sapply(group.means, function(x) x[[1]]), 
    sapply(group.means, function(x) x[[2]]),
    sapply(p.vals, function(x) x[[1]]),
    sapply(group.means, function(x) x[[3]]),
    sapply(p.vals, function(x) x[[2]])
    )
  )

tab.11[, group.2.less.1 := as.numeric(V3) - as.numeric(V2)]
tab.11[, pval.2.less.1 := V4]
tab.11[, group.3.less.2 := as.numeric(V5) - as.numeric(V3)]
tab.11[, pval.3.less.2 := V6]

saveRDS(tab.11, file = paste0(tabDir, "table11.Rds"))

##########################################################################################################
## Table 13: Balance Test

# covariates to test
covariate.list <- c("female","white","asian","black","hispanic","freelu","redlu","lep","esl.not.lep","sped",
  "grade8","grade9","grade10","grade11","grade97","age_syep_rec","zread8_rec","zmath8_rec")

covariate.names <- c("Female", "White", "Asian", "Black", "Hispanic", "Free Lunch", "Reduced Lunch", "LEP", "ESL Not LEP", "Spec Ed",
  "Grade 8", "Grade 9", "Grade 10", "Grade 11", "Alt Grade", "Age", "Zread", "Zmath")

# regress each characteristic on lottery dummies and indicator for winning
lottery.test.by.cov <- function(dt, covariate){

  reg.OLS(dt, covariate, c("select"), fixed.effects = c("cbo.year.fe"), cluster.group = c("cbo.year.fe"))

}

# test each covariate
models.lottery.test <- lapply(covariate.list, lottery.test.by.cov, dt = dt.time.varying[year %in% seq(2005,2008) & apply == 1 & app.drop.flag == 0])

## Output

# extract coefficients and standard errors
lottery.test.coefs <- round(unlist(lapply(models.lottery.test, function(x) x$coefficients[rownames(x$coefficients) == "select"])), 3)
lottery.test.se <- round(unlist(lapply(models.lottery.test, function(x) diag(x$clustervcv)[names(diag(x$clustervcv)) == "select"]^0.5)), 3)
lottery.test.pval <- round(unlist(lapply(models.lottery.test, function(x) x$pval[names(x$pval) == "select"])), 2)

# build table
tab.lottery.test.by.cov <- as.data.table(cbind(covariate.names, lottery.test.coefs, lottery.test.se, lottery.test.pval))

saveRDS(tab.lottery.test.by.cov, file = paste0(tabDir, "table13.Rds"))

##########################################################################################################
## Table 14: Falsification Test

fixed.effects.table.14 <- c("cbo.year.fe","grade.fe","cohort.fe")

covariates.falsification <- rhs.covariates

covariates.falsification <- setdiff(rhs.covariates, c("lep","esl.not.lep","sped","age_syep_rec","age_flag", "zread8_rec","zread8_flag","zmath8_rec","zmath8_flag"))

# regress selection in period t lottery on outcomes in period t-1
itt.ols.lag.list <- lapply(paste0(outcome.list, ".lag.1"), reg.OLS, 
  dt = dt.time.varying[apply == 1 & app.drop.flag == 0 & year %in% seq(2005, 2008) & obs.year.drop.flag == 0], 
  regressors = c("select"), fixed.effects = c("cbo.year.fe","grade.fe","cohort.fe"), cluster.group = cluster)

table.14 <- stargazer(itt.ols.lag.list)

saveRDS(table.14, file = paste0(tabDir, "table14.Rds"))

##########################################################################################################
## Table A2: Match Probability Regressed on Selection

model.A2 <- reg.OLS(dt.applications.raw[syepyear %in% 2006:2008 & app.drop.flag == 0 & first.app.flag.app.raw == 1], 
  "matched", c("select"), fixed.effects = c("app.fe.grade","app.fe.lottery"), cluster = c("app.fe.lottery"))

tab.A2 <- stargazer(model.A2)

saveRDS(tab.A2, file = paste0(tabDir, "tableA2.Rds"))

# save p-value for in-text placement
model.A2.p.val <- model.A2$cpval

##########################################################################################################
## Table A4: Attrition in Year Following SYEP

# flag alternative special program
dt.time.varying[grade.prev %in% c(96,97), grade2.prev := 100]
dt.time.varying[!grade.prev %in% c(96,97), grade2.prev := grade.prev]

# attrition := 1 - in student files
dt.time.varying[, attrite := 1 - in.student.files]

# attrition regression - all grades
attrition.all <- reg.OLS(dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & year %in% 2006:2009 & grade.prev != 0], 
  "attrite", c("select.prev"), fixed.effects = fixed.effects.apps, cluster.group = cluster)

# attrition regressions by grade
attrition.reg.by.grade <- function(grade.val){

  reg.OLS(dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & grade2.prev == grade.val & year %in% 2006:2009], 
    "attrite", c("select.prev"), fixed.effects = setdiff(fixed.effects.apps, "grade.prev.fe"), cluster.group = cluster)

}

grade.list <- c(8,9,10,11,100)

attrition.reg.by.grade.list <- lapply(grade.list, attrition.reg.by.grade)

list.attrition.models <- attrition.reg.by.grade.list
list.attrition.models[[length(grade.list) + 1]] <- attrition.all

table.a4 <- stargazer(list.attrition.models)

saveRDS(table.a4, file = paste0(tabDir, "tableA4.Rds"))

##########################################################################################################
## Table A6: Comparing ITT Effect Sizes

vars.for.comp <- c("black", "freelu", "zread8_rec")

# extract coefficients for select variables and treatment effect
tab.comp.select.var <- as.data.table(sapply(itt.ols.list, function(x) coef(x)[c("select.prev",vars.for.comp)]))
colnames(tab.comp.select.var) <- outcome.list

tab.comp.select.var$var.name <- c("select.prev", vars.for.comp)

# compute treatment effect as fraction of each selected coefficient
tab.comp.select.var.l <- dcast(melt(tab.comp.select.var, id.vars = "var.name"), variable ~ var.name)

invisible(lapply(vars.for.comp, function(x) tab.comp.select.var.l[, (paste0("frac.", x)) := get("select.prev") / abs(get(x))]))

tab.comp.select.var.l[, var.name := variable]
tab.comp.select.var.l[, variable := NULL]

tab.comp.select.var.out <- dcast(melt(tab.comp.select.var.l, id.vars = "var.name"), variable ~ var.name)

tab.comp.select.var.out[variable == "select.prev", sort.order := 1]
tab.comp.select.var.out[variable != "select.prev", sort.order := 2]
tab.comp.select.var.out[grep("frac", variable), sort.order := 3]

tab.comp.select.var.out <- tab.comp.select.var.out[order(sort.order)]

tab.comp.select.var.out[, variable := as.character(variable)]

saveRDS(tab.comp.select.var.out, file = paste0(tabDir, "tableA6.Rds"))

##########################################################################################################
## Table A7: Comparing TOT Effect Sizes

vars.for.comp <- c("black", "freelu", "zread8_rec")

# extract coefficients for select variables and treatment effect
tab.comp.select.var <- as.data.table(sapply(tot.2sls.list, function(x) coef(x)[c(paste0("`", treatment.var, "(fit)`"), vars.for.comp)]))
colnames(tab.comp.select.var) <- outcome.list

tab.comp.select.var$var.name <- c(treatment.var, vars.for.comp)

# compute treatment effect as fraction of each selected coefficient
tab.comp.select.var.l <- dcast(melt(tab.comp.select.var, id.vars = "var.name"), variable ~ var.name)

invisible(lapply(vars.for.comp, function(x) tab.comp.select.var.l[, (paste0("frac.", x)) := get(treatment.var) / abs(get(x))]))

tab.comp.select.var.l[, var.name := variable]
tab.comp.select.var.l[, variable := NULL]

tab.comp.select.var.out <- dcast(melt(tab.comp.select.var.l, id.vars = "var.name"), variable ~ var.name)

tab.comp.select.var.out[variable == treatment.var, sort.order := 1]
tab.comp.select.var.out[variable != treatment.var, sort.order := 2]
tab.comp.select.var.out[grep("frac", variable), sort.order := 3]

tab.comp.select.var.out <- tab.comp.select.var.out[order(sort.order)]

tab.comp.select.var.out[, variable := as.character(variable)]

saveRDS(tab.comp.select.var.out, file = paste0(tabDir, "tableA7.Rds"))

##########################################################################################################
## Table A11: Joint Test of Zero Treatment (Outcomes 1 - 6)

# test restricted model on multiple covariates by year
mvreg.outcomes <- function(dt, LHS.names, RHS.full, RHS.restrict){
  
  Y.outcome <- as.matrix(dt[, LHS.names, with = FALSE])
  
  model <- lm(as.formula(paste0("Y.outcome ~ ", RHS.full)), data = dt)
  
  model.restricted <- lm(as.formula(paste0("Y.outcome ~ ", RHS.restrict)), data = dt)
  
  anova.out <- anova(model, model.restricted)
  
  return(anova.out)
}

# RHS for full and restricted models
RHS.full <- paste0(c("select.prev", regressors.10, fixed.effects.10), collapse = "+")
RHS.restrict <- paste0(c(regressors.10, fixed.effects.10), collapse = "+")

tab.A11.pooled.anova <- mvreg.outcomes(dt.time.varying[apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0], outcome.list[1:6], RHS.full, RHS.restrict)
tab.A11.1.anova <- mvreg.outcomes(dt.time.varying[het.effects.group.1 == 1], outcome.list[1:6], RHS.full, RHS.restrict)
tab.A11.2.anova <- mvreg.outcomes(dt.time.varying[het.effects.group.2 == 1], outcome.list[1:6], RHS.full, RHS.restrict)
tab.A11.3.anova <- mvreg.outcomes(dt.time.varying[het.effects.group.3 == 1], outcome.list[1:6], RHS.full, RHS.restrict)

# extract F statistics and p-vals and arrange in table
table.A11 <- as.data.table(rbind(
  c(tab.A11.pooled.anova[[5]][[2]], tab.A11.1.anova[[5]][[2]], tab.A11.2.anova[[5]][[2]], tab.A11.3.anova[[5]][[2]]),
  c(tab.A11.pooled.anova[[8]][[2]], tab.A11.1.anova[[8]][[2]], tab.A11.2.anova[[8]][[2]], tab.A11.3.anova[[8]][[2]])
  )
  )
colnames(table.A11) <- c("pooled","group.1", "group.2", "group.3")

table.A11[, stat.name := c("F-Stat", "P-Value")]

saveRDS(table.A11, file = paste0(tabDir, "tableA11.Rds"))

##########################################################################################################
## Misc In-Text Numbers from Draft

## P-Value on ITT Effect Column 7

table.6.col.7.pval <- itt.p.vals[[7]]

## P-Value on Table A2

table.A2.pval <- model.A2.p.val

##########################################################################################################
## Memory Management for Expected Benefit Analysis

# merge application numbers into time-varying set
dt.time.varying[dt.time.invariant, on = c("sid"), num.app.incl.drop := num.app.incl.drop]

dt.time.varying[, num.app.incl.drop.nonzero := as.numeric(num.app.incl.drop>0)]

# dt for estimating treatment effects: outcomes in period t, applications in period t-1
dt.est <- dt.time.varying[year %in% seq(2006,2009) & apply.prev == 1 & app.drop.flag.prev == 0 & obs.year.drop.flag == 0]

# dt of appliers and comparable non-appliers in period t
dt.all <- dt.time.varying[year %in% seq(2005,2008) & grade.drop == 0 & grade != 0]

# define applier and non-applier (ignore impermissible applications)
dt.all[, applier.flag := as.numeric(apply == 1 & app.drop.flag == 0)]
dt.all[apply == 1 & app.drop.flag == 1, applier.flag := NA]

rm(list = c("dt.time.invariant", "dt.time.varying"))
gc()

##########################################################################################################
## Create Interactions of Covariates and Treatment

# list of variables to interact
var.list.to.inter <- c(rhs.covariates, "num.app.incl.drop", "cum.num.apps", "cum.num.apps.incl.drop", "apply.next")

setDT(dt.est)

invisible(lapply(var.list.to.inter, create.inter, dt = dt.est, var.1 = "worked.prev"))
invisible(lapply(var.list.to.inter, create.inter, dt = dt.est, var.1 = "select.prev"))

##########################################################################################################
## Table A8: Full Treatment Interactions

## A8a: Het Effects 2SLS

rhs.covariates.full.inter <- setdiff(rhs.covariates, c("miss_gender", "age_flag"))

# 2SLS on each outcome (TOT model) with full interactions
tot.2sls.full.inter.list <- lapply(outcome.list, reg.2SLS, dt = dt.est, 
  treatment.var = treatment.var, instrument.var = instrument.var, 
  regressors = rhs.covariates.full.inter, fixed.effects = fixed.effects.apps, 
  interact.with.treatment = rhs.covariates.full.inter, cluster.group = cluster)

table.A8a <- stargazer(tot.2sls.full.inter.list)

saveRDS(table.A8a, file = paste0(tabDir, "tableA8a.Rds"))

# extract R2 from projection model
tab.A8a.R2 <- sapply(tot.2sls.full.inter.list, function(x) summary(x)$P.r.squared)

saveRDS(tab.A8a.R2, paste0(tabDir, "tableA8a_R2.Rds"))

## A8b: F-Tests on Joint Restriction that All Endogenous Variable Coefficients are Zero

f.test.full.inter.list <- lapply(tot.2sls.full.inter.list, function(x) waldtest(x, setdiff(x$endovars, "worked.prev")))

## Formatting and Output

tab.f.test.full.inter <- as.data.table(rbind(
  sapply(f.test.full.inter.list, function(x) x[["F"]]), 
  sapply(f.test.full.inter.list, function(x) x[["p.F"]])
  )
  )

colnames(tab.f.test.full.inter) <- c(outcome.list)
tab.f.test.full.inter$stat.names <- c("F-Stat","P-Value")
      
saveRDS(tab.f.test.full.inter, file = paste0(tabDir, "tableA8b.Rds"))

rm(list = c("tot.2sls.full.inter.list", "f.test.full.inter.list"))
gc()

##########################################################################################################
## Compute Expected Benefits

# set covariates on which to predict
covs.for.prediction <- rhs.covariates

setDT(dt.all)

invisible(lapply(
  outcome.list, 
  estimate.predict.marginal.benefit, 
  dt.est = dt.est, dt.predict = dt.all, 
  treatment.var = treatment.var, instrument.var = instrument.var, 
  regressors = covs.for.prediction, fixed.effects = fixed.effects.apps
)
)

setDT(dt.est)

invisible(lapply(
  outcome.list, 
  estimate.predict.marginal.benefit, 
  dt.est = dt.est, dt.predict = dt.est, 
  treatment.var = treatment.var, instrument.var = instrument.var, 
  regressors = covs.for.prediction, fixed.effects = fixed.effects.apps
)
)

#########################################################################################################
## Table 12: Average Expected Benefit by Participation History

table.12 <- dt.est[!is.na(het.effects.group), lapply(.SD, mean, na.rm = TRUE), .SDcols = paste0("eb.",outcome.list), by = .(het.effects.group)][order(het.effects.group)]

saveRDS(table.12, file = paste0(tabDir, "table12.Rds"))

#########################################################################################################
## Bootstrapping Average EB Standard Errors

rerun.bootstrap.LATE <- 0
rerun.bootstrap.EB <- 0

import.bind.LATE <- 0
import.bind.EB <- 0

#= LATE =#

bootstrap.B <- 1000
n.iter.split <- 10

if (rerun.bootstrap.LATE == 1){
  
  set.seed(5678)
  seed.vec <- sample(1:2^15, n.iter.split)
  
  for (iter.split in 1:n.iter.split){
    
    print(paste("LATE Bootstrap Split Block", iter.split, "of", n.iter.split, ", Total Iterations:", bootstrap.B, ", Seed", seed.vec[iter.split]))
    set.seed(seed.vec[iter.split])
    
    bootstrap.B.iter.split <- floor(bootstrap.B / n.iter.split)
    
    dt.iter.split <- bootstrap.LATE.SE(dt.est, outcome.list, bootstrap.B.iter.split, treatment.var, instrument.var, rhs.covariates, fixed.effects.apps, 
      block.cluster = TRUE, run.parallel = FALSE, n.cores = 18)
    
    saveRDS(dt.iter.split, paste0(cleanDir, "/dt_bootstrap_LATE_cluster_", bootstrap.B, "_", n.iter.split, "_", iter.split, ".Rds"))
    
    rm("dt.iter.split")
    gc()
  }
  
}

if (import.bind.LATE == 1){
 
  dt.bootstrap.LATE.list <- list()
  
  for (iter.split in 1:n.iter.split) {

    dt.bootstrap.LATE.list[[iter.split]] <- readRDS(paste0(cleanDir, "/dt_bootstrap_LATE_cluster_", bootstrap.B, "_", n.iter.split, "_", iter.split, ".Rds"))
  
  }
  
  dt.bootstrap.LATE <- rbindlist(dt.bootstrap.LATE.list)
  
  # save
  saveRDS(dt.bootstrap.LATE, paste0(cleanDir, "/dt_bootstrap_LATE_cluster.Rds"))
   
}

#= Expected Benefit by Covariates =#

bootstrap.B <- 1000
n.iter.split <- 40

if (rerun.bootstrap.EB == 1){
  
  set.seed(9112)
  seed.vec <- sample(1:2^15, n.iter.split)
  
  for (iter.split in 1:n.iter.split){
    
    print(paste("EB Bootstrap Split Block", iter.split, "of", n.iter.split, ", Total Iterations:", bootstrap.B, ", Seed", seed.vec[iter.split]))
    set.seed(seed.vec[iter.split])
    
    bootstrap.B.iter.split <- floor(bootstrap.B / n.iter.split)
    
    dt.iter.split <- bootstrap.EB.stat.SE(dt.est, dt.all, outcome.list, bootstrap.B.iter.split, treatment.var, instrument.var, 
      rhs.covariates, fixed.effects.apps, predict.coefs = NULL, 
      by.var.list = list(c("female"), c("female","white","black","asian","hispanic"), c("num.app.incl.drop"), c("num.app.incl.drop.nonzero")), 
      by.var.list.label = list(c("sex"), c("sex by race"), c("number apps"), c("ever apply")), 
      block.cluster = TRUE, apply.only = FALSE, run.parallel = FALSE, n.cores = 18)
    
    saveRDS(dt.iter.split[[1]], paste0(cleanDir, "/dt_bootstrap_EB_apply_cluster_", bootstrap.B, "_", n.iter.split, "_", iter.split, ".Rds"))
    saveRDS(dt.iter.split[[2]], paste0(cleanDir, "/dt_bootstrap_EB_nonapply_cluster_", bootstrap.B, "_", n.iter.split, "_", iter.split, ".Rds"))
    
    rm("dt.iter.split")
    gc()
  }
  
}

if (import.bind.EB == 1){
  
  dt.bootstrap.EB.1.1.list <- list()
  dt.bootstrap.EB.1.2.list <- list()
  
  for (iter.split in 1:n.iter.split){
    dt.bootstrap.EB.1.1.list[[iter.split]] <- readRDS(paste0(cleanDir, "/dt_bootstrap_EB_apply_cluster_", bootstrap.B, "_", n.iter.split, "_", iter.split, ".Rds"))
    dt.bootstrap.EB.1.2.list[[iter.split]] <- readRDS(paste0(cleanDir, "/dt_bootstrap_EB_nonapply_cluster_", bootstrap.B, "_", n.iter.split, "_", iter.split, ".Rds"))
    
    # correct iterations to account for split processing
    dt.bootstrap.EB.1.1.list[[iter.split]][, iter := iter + (iter.split - 1)*floor(bootstrap.B / n.iter.split)]
    dt.bootstrap.EB.1.2.list[[iter.split]][, iter := iter + (iter.split - 1)*floor(bootstrap.B / n.iter.split)]
  }
  
  dt.bootstrap.EB.1.1 <- rbindlist(dt.bootstrap.EB.1.1.list)
  dt.bootstrap.EB.1.2 <- rbindlist(dt.bootstrap.EB.1.2.list)
  
  # save (pooled only)
  saveRDS(dt.bootstrap.EB.1.1[by.var.label == "pooled"], paste0(cleanDir, "/dt_bootstrap_EB_apply_cluster.Rds"))
  saveRDS(dt.bootstrap.EB.1.2[by.var.label == "pooled"], paste0(cleanDir, "/dt_bootstrap_EB_nonapply_cluster.Rds"))
  
}

#########################################################################################################
## Tables 8, A9: Expected Benefit Distribution

# import bootstrapped LATE
dt.bootstrap.LATE <- readRDS(paste0(cleanDir,"/dt_bootstrap_LATE_cluster.Rds"))

# import bootstrapped EB
dt.bootstrap.EB.apply <- readRDS(paste0(cleanDir, "/dt_bootstrap_EB_apply_cluster.Rds"))
dt.bootstrap.EB.nonapply <- readRDS(paste0(cleanDir, "/dt_bootstrap_EB_nonapply_cluster.Rds"))

# merge LATE bootstrap into applier EB bootstrap
dt.bootstrap.LATE[, iter := .I]

dt.bootstrap.EB.apply <- merge(dt.bootstrap.EB.apply, dt.bootstrap.LATE, by = c("iter"))

# for each bootstrap iteration and outcome, compute P50-P10 and P90-P10
invisible(lapply(outcome.list, function(x) dt.bootstrap.EB.apply[, (paste0("p50.less.p10.eb.", x)) := get(paste0("p50.eb.", x)) - get(paste0("p10.eb.", x))]))
invisible(lapply(outcome.list, function(x) dt.bootstrap.EB.nonapply[, (paste0("p50.less.p10.eb.", x)) := get(paste0("p50.eb.", x)) - get(paste0("p10.eb.", x))]))

invisible(lapply(outcome.list, function(x) dt.bootstrap.EB.apply[, (paste0("p90.less.p50.eb.", x)) := get(paste0("p90.eb.", x)) - get(paste0("p50.eb.", x))]))
invisible(lapply(outcome.list, function(x) dt.bootstrap.EB.nonapply[, (paste0("p90.less.p50.eb.", x)):= get(paste0("p90.eb.", x)) - get(paste0("p50.eb.", x))]))

stat.names <- grep("^mean|^p01|^p10|^p50|^p90|^p99|^LATE", colnames(dt.bootstrap.EB.apply), value = TRUE)

## 8a: Appliers Only

table.8a.est <- compute.EB.stats(dt.all[apply == 1 & app.drop.flag == 0], grep(paste0("^eb\\."), colnames(dt.all), value = TRUE))

# compute P50-P10 and P90-P10
invisible(lapply(outcome.list, function(x) table.8a.est[, (paste0("p50.less.p10.eb.", x)) := get(paste0("p50.eb.", x)) - get(paste0("p10.eb.", x))]))
invisible(lapply(outcome.list, function(x) table.8a.est[, (paste0("p90.less.p50.eb.", x)) := get(paste0("p90.eb.", x)) - get(paste0("p50.eb.", x))]))

# append LATE to applier EB stats
LATE.est <- matrix(unlist(LATE.2sls.list), nrow = 1)
colnames(LATE.est) <- paste0("LATE.", outcome.list)

table.8a.est.LATE <- cbind(table.8a.est, LATE.est)

# generate standard errors and p-values from bootstrapped results
table.8a <- gen.bootstrap.se.pvals(dt.bootstrap.EB.apply, table.8a.est.LATE, stat.names)

saveRDS(table.8a, file = paste0(tabDir, "table8a.Rds"))

## 8b: Non-Appliers Only

table.8b.est <- compute.EB.stats(dt.all[apply == 0], grep(paste0("^eb\\."), colnames(dt.all), value = TRUE))

# compute P50-P10 and P90-P10
invisible(lapply(outcome.list, function(x) table.8b.est[, (paste0("p50.less.p10.eb.", x)) := get(paste0("p50.eb.", x)) - get(paste0("p10.eb.", x))]))
invisible(lapply(outcome.list, function(x) table.8b.est[, (paste0("p90.less.p50.eb.", x)) := get(paste0("p90.eb.", x)) - get(paste0("p50.eb.", x))]))

# generate standard errors and p-values from bootstrapped results
table.8b <- gen.bootstrap.se.pvals(dt.bootstrap.EB.nonapply, table.8b.est, grep("^LATE", stat.names, value = TRUE, invert = TRUE))

saveRDS(table.8b, file = paste0(tabDir, "table8b.Rds"))

## A9: Appliers Less Non-Appliers

table.8a[, dist.name := "apply"]
table.8b[, dist.name := "nonapply"]

invisible(lapply(c("estimate", "se"), function(x) table.8a[, (paste0(x, ".apply")) := get(x)]))
invisible(lapply(c("estimate", "se"), function(x) table.8b[, (paste0(x, ".nonapply")) := get(x)]))

# compute difference in each statistic (as fraction of applier number)
table.A9.est <- merge(table.8a[!like(stat.name, "less"), .(outcome.name, stat.name, estimate.apply, se.apply)], table.8b[!like(stat.name, "less"), .(outcome.name, stat.name, estimate.nonapply, se.nonapply)],
  by = c("outcome.name", "stat.name"))

table.A9.est[, diff.est := (estimate.apply - estimate.nonapply)]
table.A9.est[, diff.est.perc.apply := diff.est / estimate.apply]

### P-Values Testing Difference is Zero

# melt bootstrap iterations by statistic type and iteration
dt.bootstrap.EB.apply.l <- melt(dt.bootstrap.EB.apply[by.var.label == "pooled", c("iter", grep("^p|^mean", colnames(dt.bootstrap.EB.apply), value = TRUE)), with = FALSE], 
  id.vars = c("iter"), variable.factor = FALSE)
dt.bootstrap.EB.apply.l[, estimate.i.apply := value]

dt.bootstrap.EB.nonapply.l <- melt(dt.bootstrap.EB.nonapply[by.var.label == "pooled", c("iter", grep("^p|^mean", colnames(dt.bootstrap.EB.nonapply), value = TRUE)), with = FALSE], 
  id.vars = c("iter"), variable.factor = FALSE)
dt.bootstrap.EB.nonapply.l[, estimate.i.nonapply := value]

dt.bootstrap.EB.l.merge <- merge(dt.bootstrap.EB.apply.l[!like(variable, "less"), .(iter, variable, estimate.i.apply)], dt.bootstrap.EB.nonapply.l[!like(variable, "less"), .(iter, variable, estimate.i.nonapply)], 
  by = c("iter","variable"))

# bootstrap statistic is difference in estimates
dt.bootstrap.EB.l.merge[, diff.i := estimate.i.apply - estimate.i.nonapply]

# extract outcome and statistic
dt.bootstrap.EB.l.merge[substr(variable,1,4) == "mean", c("stat.name", "outcome.name") := list("mean.eb", substr(variable, 9, nchar(variable)))]
dt.bootstrap.EB.l.merge[substr(variable,1,4) != "mean", c("stat.name", "outcome.name") := list(substr(variable, 1, 6), substr(variable, 8, nchar(variable)))]

# merge in estimates
dt.bootstrap.EB.l.merge[, variable := NULL]
dt.bootstrap.EB.l.merge[table.A9.est, on = c("stat.name", "outcome.name"), c("diff.est", "diff.est.perc.apply") := list(diff.est, diff.est.perc.apply)]

# find bootstrap standard errors of difference
dt.bootstrap.EB.l.merge[, diff.se := var(diff.i)^0.5, by = .(outcome.name, stat.name)]

# compute bootstrap t-stats
dt.bootstrap.EB.l.merge[, t.stat.i := abs(diff.i - diff.est) / diff.se]

# compute t-stat under H0 = 0 (of true distribution, replicated on each iteration for convenience)
dt.bootstrap.EB.l.merge[, t.stat.H0 := abs(diff.est - 0) / diff.se]

# p-val = fraction of distribution that t-stat under H0 = 0 is above (abs value)
p.vals.diff.l <- dt.bootstrap.EB.l.merge[, .(p.val = mean(as.numeric(t.stat.H0 <= t.stat.i))), by = .(outcome.name, stat.name)]

table.A9 <- merge(table.A9.est, p.vals.diff.l)

saveRDS(table.A9, file = paste0(tabDir, "tableA9.Rds"))

#########################################################################################################
## Table A10: 90-10 EB Quantile Covariate Differences

covs.to.test <- c("female", "black", "asian", "hispanic", "freelu", "redlu", "lep", "esl.not.lep", "sped", "age_syep_rec", "zread8", "zmath8")
cov.miss.flag <- c("miss_gender", "miss_ethn", "miss_ethn", "miss_ethn", "missfrl", "missfrl", "0", "0", "0", "age_flag", "zread8_flag", "zmath8_flag")

# flag decile of EB for each outcome
flag.decile <- function(dt, eb.var, by.vars = NULL){
  dt[, paste0(eb.var, ".decile") := cut(get(eb.var), unique(quantile(get(eb.var), probs = seq(0, 1, 0.1), na.rm = TRUE)), include.lowest = TRUE, labels = FALSE), by = by.vars]
}

dt.all[apply == 1 & app.drop.flag == 0, dist.name := "applier"]
dt.all[apply == 0, dist.name := "nonapplier"]

setDT(dt.all)

invisible(lapply(paste0("eb.", outcome.list), flag.decile, dt = dt.all, by.vars = c("dist.name")))

## A10a: Appliers Only

table.A10a.list <- lapply(paste0("eb.", outcome.list, ".decile"), 
  function(eb.name) mapply(test.mean.diffs, covariate = covs.to.test, covariate.miss.flag = cov.miss.flag, 
    MoreArgs = list(dt.1 = dt.all[dist.name == "applier" & get(eb.name) == 10], dt.2 = dt.all[dist.name == "applier" & get(eb.name) == 1]), SIMPLIFY = FALSE)
)

## A10b: Non-Appliers Only

table.A10b.list <- lapply(paste0("eb.", outcome.list, ".decile"), 
  function(eb.name) mapply(test.mean.diffs, covariate = covs.to.test, covariate.miss.flag = cov.miss.flag, 
    MoreArgs = list(dt.1 = dt.all[dist.name == "nonapplier" & get(eb.name) == 10], dt.2 = dt.all[dist.name == "nonapplier" & get(eb.name) == 1]), SIMPLIFY = FALSE)
)

## Package into long tables

package.long <- function(list.A10){

  table.A10 <- rbindlist(lapply(seq_along(list.A10), 
    function(outcome.i) {

      dt.i <- rbindlist(list.A10[[outcome.i]])
      colnames(dt.i) <- c("mean.90", "mean.10", "diff", "p.val")

      dt.i[, outcome.name := outcome.list[[outcome.i]]]
      dt.i[, cov.name := covs.to.test]

      return(dt.i)
    }
  )
  )

  return(table.A10)
}

table.A10a.long <- package.long(table.A10a.list)

saveRDS(table.A10a.long, file = paste0(tabDir, "tableA10a.Rds"))

table.A10b.long <- package.long(table.A10b.list)

saveRDS(table.A10b.long, file = paste0(tabDir, "tableA10b.Rds"))