##########################################################################################################
## Program: buildBaseDT
## Description: builds time-varying and time-invariant base datasets for analysis
##########################################################################################################

print("Building master datasets for analysis")

##########################################################################################################
## Import

# student files
dt.student.files <- readRDS(paste0(cleanDir, "/dt_student_files.Rds"))

# application data
dt.applications.raw <- readRDS(paste0(cleanDir, "/dt_syep_applications.Rds"))

# regents data
dt.regents <- readRDS(paste0(cleanDir, "/dt_regents_scores_0409.Rds"))

##########################################################################################################
## Create Time-Varying Dataset

## Set Application File

dt.applications <- dt.applications.raw[matched == 1]

rm("dt.applications.raw")
gc()

## Merge Regents Outcomes and Time-Varying Student Covariates

# flag student existing in student files
dt.student.files[, in.student.files := 1]

# flag students who ever show up in regents files
dt.regents[, ever.regents := 1]

exam.list <- c("escore","bscore","cscore","esscore","flscore","pscore","gscore","mscore","mbscore","uscore","geoscore","algscore")

dt.time.varying.a <- merge(dt.student.files, 
  dt.regents[, mget(c("sid","academic.year","ever.regents", grep("n_attmpt|ever_attempt|evr_pass65|n_pass55|n_pass65|n_pass75|ave_score2zyr|frac", colnames(dt.regents), value = TRUE), exam.list))],
  by.x = c("sid", "year"), by.y = c("sid", "academic.year"), all.x = TRUE)

rm(list = c("dt.regents", "dt.student.files"))
gc()

## Merge Applications into Time-Varying Student Covariates

# specify drop reason flags
app.drop.flags <- c("app.drop.flag", "has.dupe.app", "wia.vul.drop", "provider.drop", "pctsel.drop")

# merge contemporaneous application into student files (if duplicates, merge in selected application only)
dt.applications[, apply := 1]

dt.time.varying.b <- merge(dt.time.varying.a, 
  dt.applications[has.dupe.app == 0 | dupe.keep == 1, mget(c("sid", "syepyear", "apply", "select", "worked", "provider.clean", app.drop.flags))], 
  by.x = c("sid", "year"), by.y = c("sid", "syepyear"), all = TRUE)

rm("dt.time.varying.a")
gc()

# merge last year application into student files
dt.applications[, year := syepyear + 1]

lag.application.var <- function(dt,var.name){

  dt[, (paste0(var.name,".prev")) := dt[, var.name, with = FALSE]]

}

app.vars.to.lag <- c("provider.clean", "apply", "worked", "select", "has.dupe.app", "wia.vul.drop", "provider.drop", "pctsel.drop", "app.drop.flag")

setDT(dt.applications)

invisible(lapply(app.vars.to.lag, lag.application.var, dt = dt.applications))

# specify drop reason flags for previous year application
app.drop.flags.prev <- paste0(app.drop.flags, ".prev")

dt.time.varying <- merge(dt.time.varying.b, 
  dt.applications[has.dupe.app == 0 | dupe.keep == 1, mget(c("sid", "year", "apply.prev", "select.prev", "worked.prev", "provider.clean.prev", app.drop.flags.prev))], 
  by = c("sid", "year"), all = TRUE)

rm(list = c("dt.time.varying.b", "dt.applications"))
gc()

dt.time.varying[is.na(in.student.files), in.student.files := 0]
dt.time.varying[is.na(ever.regents), ever.regents := 0]

## Set Regents Outcomes that Depend on Count to 0 if NA

list.to.zero <- grep("n_attmpt|ever_attempt|evr_pass65|n_pass55|n_pass65|n_pass75", colnames(dt.time.varying), value = TRUE)

invisible(lapply(list.to.zero, function(x) dt.time.varying[is.na(get(x)), (x) := 0]))

## Identify Non-Continuous Observations

setkey(dt.time.varying, sid, year)

dt.time.varying[, year.noncts := as.numeric((year - 1) != shift(year, type = "lag") & !is.na(shift(year, type = "lag"))), by = .(sid)]

## Lag Grade

setkey(dt.time.varying, sid, year)

dt.time.varying[, grade.prev := shift(grade, type = "lag"), by = .(sid)]
dt.time.varying[year.noncts == 1, grade.prev := NA]

# flag application in grade 12 or 7 to drop
dt.time.varying[, grade.drop := as.numeric(grade %in% c(1, 2, 3, 4, 5, 6, 7, 12, 95, 98, 88, 99) & !is.na(grade))]
dt.time.varying[, grade.drop.prev := as.numeric(grade.prev %in% c(1, 2, 3, 4, 5, 6, 7, 12, 95, 98, 88, 99) & !is.na(grade.prev))]

dt.time.varying[apply == 1 & grade.drop == 1 & app.drop.flag == 0, app.drop.flag := 1]
dt.time.varying[apply.prev == 1 & grade.drop.prev == 1 & app.drop.flag.prev == 0, app.drop.flag.prev := 1]

## Flag First Year of Application and if Student Ever Applies (2005-2008)

# flag if student ever applies
dt.time.varying[, ever.apply.0508 := max(apply[app.drop.flag == 0 & year < 2009], na.rm = TRUE), by = .(sid)]
dt.time.varying[, ever.apply.0508.incl.drop := max(apply[year < 2009], na.rm = TRUE), by = .(sid)]

dt.time.varying[is.infinite(ever.apply.0508), ever.apply.0508 := 0]
dt.time.varying[is.infinite(ever.apply.0508.incl.drop), ever.apply.0508.incl.drop := 0]

# flag first year of application
dt.time.varying[ever.apply.0508 == 1, first.app.year := min(year[apply == 1 & app.drop.flag == 0 & year < 2009], na.rm = TRUE), by = .(sid)]
dt.time.varying[ever.apply.0508.incl.drop == 1, first.app.year.incl.drop := min(year[apply == 1 & year < 2009], na.rm = TRUE), by = .(sid)]

## Create Cohort of Initial Application

# create grade-for-cohort variable
dt.time.varying[apply.prev == 1, grade.cohort := grade.prev]

# interpolate last grade if missing
dt.time.varying[is.na(grade.cohort) & apply.prev == 1 & (!grade %in% c(88, 95, 96, 97, 98, 99)), grade.cohort := grade - 1]

# set grad.cohort to missing if impermissible value
dt.time.varying[grade.cohort %in% c(1, 2, 3, 4, 5, 6, 12, 88, 95, 98), grade.cohort := NA]

setkey(dt.time.varying, sid)

# find earliest year with permissible cohort
dt.time.varying[, any.valid.grade.cohort := max(as.numeric(!is.na(grade.cohort))), by = .(sid)]
dt.time.varying[any.valid.grade.cohort == 1, min.year.cohort := min(year[!is.na(grade.cohort)]), by = .(sid)]

# cohort is earliest permissible value
dt.time.varying[any.valid.grade.cohort == 1, grade.cohort.all := grade.cohort[year == min.year.cohort], by = .(sid)]

# cohort = grade x year
dt.time.varying[any.valid.grade.cohort == 1, cohort := paste0(grade.cohort.all, "-", first.app.year)]

## Flag Missing Cohort

dt.time.varying[, cohort.missing.drop := as.numeric(is.na(cohort))]
dt.time.varying[, cohort.missing.drop.prev := as.numeric(is.na(cohort))]

# flag to drop these applications
dt.time.varying[cohort.missing.drop == 1 & app.drop.flag == 0, app.drop.flag := 1]
dt.time.varying[cohort.missing.drop.prev == 1 & app.drop.flag.prev == 0, app.drop.flag.prev := 1]

## Flag Missing Previous Grade

dt.time.varying[, grade.prev.miss := as.numeric(is.na(grade.prev))]
dt.time.varying[grade.prev.miss == 1, grade.prev := 0]

dt.time.varying[, grade.miss := as.numeric(is.na(grade))]
dt.time.varying[grade.miss == 1, grade := 0]

## Count Applications and Participation for Each Student (2005-2008)

setkey(dt.time.varying, sid, year)

dt.time.varying[is.na(apply), apply := 0]
dt.time.varying[is.na(apply.prev), apply.prev := 0]

dt.time.varying[apply == 0, worked := 0]
dt.time.varying[apply == 0, select := 0]

dt.time.varying[apply.prev == 0, worked.prev := 0]
dt.time.varying[apply.prev == 0, select.prev := 0]

dt.time.varying[is.na(app.drop.flag), app.drop.flag := 0]
dt.time.varying[is.na(app.drop.flag.prev), app.drop.flag.prev := 0]

# cumulative applications and lottery outcomes
dt.time.varying[, cum.num.apps := cumsum(apply.prev*(1 - app.drop.flag.prev)), by = .(sid)]
dt.time.varying[, cum.num.select := cumsum(select.prev*(1- app.drop.flag.prev)), by = .(sid)]
dt.time.varying[, cum.num.worked := cumsum(worked.prev*(1- app.drop.flag.prev)), by = .(sid)]

# same as above but including dropped applications
dt.time.varying[, cum.num.apps.incl.drop := cumsum(apply.prev), by = .(sid)]
dt.time.varying[, cum.num.select.incl.drop := cumsum(select.prev), by = .(sid)]
dt.time.varying[, cum.num.worked.incl.drop := cumsum(worked.prev), by = .(sid)]

lag.hist.var <- function(dt,var.name){

  dt[, (paste0(var.name, ".prev")) := shift(get(var.name), type = "lag"), by = .(sid)]
  dt[year.noncts == 1, (paste0(var.name, ".prev")) := NA]

}

# lag second (preferred) set of histories
hist.vars.to.lag <- c("cum.num.apps.incl.drop", "cum.num.select.incl.drop", "cum.num.worked.incl.drop")

setkey(dt.time.varying, sid, year)
setDT(dt.time.varying)

invisible(lapply(hist.vars.to.lag, lag.hist.var, dt = dt.time.varying))

## Create Fixed Effects

# cbo and cbo x year fixed effects, both contemporaneous and previous year
dt.time.varying[!is.na(provider.clean.prev), cbo.prev.fe := as.factor(provider.clean.prev)]
dt.time.varying[!is.na(provider.clean), cbo.fe := as.factor(provider.clean)]
dt.time.varying[!is.na(provider.clean.prev), cbo.prev.year.fe := as.factor(paste0(provider.clean.prev, year))]
dt.time.varying[!is.na(provider.clean), cbo.year.fe := as.factor(paste0(provider.clean, year))]

# cohort and cohort x year fixed effects
dt.time.varying[, cohort.fe := as.factor(cohort)]
dt.time.varying[, cohort.year.fe := paste0(cohort,year)]

# grade and year fixed effects, both contemporaneous and previous year
dt.time.varying[, grade.prev.fe := as.factor(grade.prev)]
dt.time.varying[, grade.fe := as.factor(grade)]
dt.time.varying[, year.prev.fe := as.factor(year - 1)]
dt.time.varying[, year.fe := as.factor(year)]

# grade by year fixed effects
dt.time.varying[, grade.year.prev.fe := as.factor(paste0(grade.prev, ".", year - 1))]
dt.time.varying[, grade.year.fe := as.factor(paste0(grade, ".", year))]

## Flag Group Who Applies Again

setkey(dt.time.varying, sid, year)

# flag next year
dt.time.varying[, year.next := shift(year, type = "lead"), by = .(sid)]

# flag if applies following year if continuous
dt.time.varying[, apply.next := shift(apply, type = "lead"), by = .(sid)]
dt.time.varying[year.next != year + 1, apply.next := NA]

# flag last year observed in data
dt.time.varying[, last.year.obs := max(year), by = .(sid)]

dt.time.varying[, year.next := NULL]

## Flag Observation Years to Drop (trims impermissible outcome years, reducing number of past applications to consider)

# students missing from student files or regents
dt.time.varying[, student.missing := as.numeric(in.student.files == 0)]

# missing dbn variable
dt.time.varying[, dbn.missing := as.numeric(is.na(dbn) | dbn == "")]

# combine
dt.time.varying[, obs.year.drop.flag := as.numeric(student.missing + dbn.missing > 0)]

##########################################################################################################
## Create Time-Invariant Dataset

## Extract Time-Invariant Characteristics of Students

dt.student.files.time.inv <- unique(dt.time.varying[in.student.files == 1, 
  mget(c("sid", "in.student.files", grep("\\.mode", colnames(dt.time.varying), value = TRUE), "zread8", "zread8_rec", "zread8_flag", "zmath8", "zmath8_rec", "zmath8_flag", "cohort9_year"))])

## Flag Missing 9th Grade Cohort

dt.student.files.time.inv[, cohort9_year.miss := as.numeric(is.na(cohort9_year))]
dt.student.files.time.inv[cohort9_year.miss == 1, cohort9_year := 0]

## Merge Number of Applications into Time-Invariant Dataset (applications 2005-2008)

# count applications
app.num.by.student <- dt.time.varying[year <= 2009, 
  .(
    num.app = sum(apply.prev[app.drop.flag.prev == 0 & obs.year.drop.flag == 0]), 
    num.app.incl.drop = sum(apply.prev)
    ), by = .(sid)]

# count wins
win.num.by.student <- dt.time.varying[year <= 2009, 
  .(
    num.win = sum(select.prev[app.drop.flag.prev == 0 & obs.year.drop.flag == 0]), 
    num.win.incl.drop = sum(select.prev)
    ), by = .(sid)]

# count participation
work.num.by.student <- dt.time.varying[year <= 2009, 
  .(
    num.work = sum(worked.prev[app.drop.flag.prev == 0 & obs.year.drop.flag == 0]),
    num.work.incl.drop = sum(worked.prev)
    ), by = .(sid)]

app.summary <- Reduce(function(...) merge(..., by = "sid", all = TRUE), list(app.num.by.student, win.num.by.student, work.num.by.student))

dt.time.invariant <- merge(dt.student.files.time.inv, app.summary, by = c("sid"), all.x = TRUE)

dt.time.invariant[is.na(num.app), num.app := 0]
dt.time.invariant[is.na(num.win), num.win := 0]
dt.time.invariant[is.na(num.work), num.work := 0]

dt.time.invariant[is.na(num.app.incl.drop), num.app.incl.drop := 0]
dt.time.invariant[is.na(num.win.incl.drop), num.win.incl.drop := 0]
dt.time.invariant[is.na(num.work.incl.drop), num.work.incl.drop := 0]

# flag if ever apply
dt.time.invariant[, apply := as.numeric(num.app > 0)]

## Count Observations between Grades 8-11 and 96/97 for Each Student (2005-2008)

grade.obs <- dt.time.varying[in.student.files == 1, 
  .(obs.total = sum(as.numeric(year <= 2008)), 
    obs.grade.8.11.alt = sum(as.numeric(grade %in% c(8, 9, 10, 11, 96, 97) & year <= 2008)),
    min.grade = min(grade[grade <= 12 & year <= 2008], na.rm = TRUE), 
    max.grade = max(grade[grade <= 12 & year <= 2008], na.rm = TRUE),
    n.alt.grade = sum(as.numeric(grade %in% c(96, 97) & year <= 2008))
    ), by = .(sid)]

grade.obs[is.infinite(min.grade), min.grade := NA]
grade.obs[is.infinite(max.grade), max.grade := NA]

# merge into invariant dataset
dt.time.invariant <- merge(dt.time.invariant, grade.obs, by = c("sid"), all.x = TRUE)

## Count Years and Grades Observed

year.grade.obs.flag <- dt.time.varying[in.student.files == 1, 
  .(obs.total = sum(as.numeric(year <= 2008)), 
    obs.grade.8.11.alt = sum(as.numeric(grade %in% c(8, 9, 10, 11, 96, 97) & year <= 2008)),
    min.grade = min(grade[grade <= 12 & year <= 2008], na.rm = TRUE), 
    max.grade = max(grade[grade <= 12 & year <= 2008], na.rm = TRUE),
    n.alt.grade = sum(as.numeric(grade %in% c(96, 97) & year <= 2008))
    ), by=.(sid)]

## Flag Grade and Calendar Year Observed

# grade
invisible(lapply(list(8,9,10,11,12,96,97), function(x) dt.time.varying[, (paste0("obs.grade.",x)) := as.numeric(max2(as.numeric(in.student.files == 1 & grade == x))), by = .(sid)]))

# calendar year
invisible(lapply(unique(dt.time.varying[,year]), function(x) dt.time.varying[, (paste0("obs.year.", x)) := as.numeric(max2(as.numeric(in.student.files == 1 & year == x))), by = .(sid)]))

## Summarize Outcomes Between Grades 8 and 12 (or alternate) for Students Observed in Grade 12

cross.section.outcomes <- dt.time.varying[in.student.files == 1 & obs.grade.12 == 1, 
  .(obs.cross.section = sum(as.numeric(grade %in% c(8,9,10,11,12,96,97))), 
    obs.grade.8 = max(obs.grade.8), 
    obs.grade.9 = max(obs.grade.9), 
    obs.grade.10 = max(obs.grade.10), 
    obs.grade.11 = max(obs.grade.11), 
    obs.grade.96 = max(obs.grade.96), 
    obs.grade.97 = max(obs.grade.97),
    obs.year.2005 = max(obs.year.2005), 
    obs.year.2006 = max(obs.year.2006), 
    obs.year.2007 = max(obs.year.2007), 
    obs.year.2008 = max(obs.year.2008), 
    obs.year.2009 = max(obs.year.2009),
    ever_attempt = max(ever_attempt), 
    n_attmpt = sum(n_attmpt), 
    evr_pass65 = max(evr_pass65),
    n_pass55 = sum(n_pass55), 
    n_pass65 = sum(n_pass65), 
    n_pass75 = sum(n_pass75), 
    ave_score2zyr = mean(ave_score2zyr, na.rm=TRUE)
    ), by = .(sid)]

# scale outcomes by years observed
count.outcome.list <- c("ever_attempt", "n_attmpt", "evr_pass65", "n_pass55", "n_pass65", "n_pass75")

invisible(lapply(count.outcome.list, function(x) cross.section.outcomes[, (paste0(x, ".scale")) := get(x) / sum(obs.cross.section), by = .(sid)]))

# compute fraction passed at each level
cross.section.outcomes[ever_attempt == 1, frac_pass55 := n_pass55 / n_attmpt]
cross.section.outcomes[ever_attempt == 1, frac_pass65 := n_pass65 / n_attmpt]
cross.section.outcomes[ever_attempt == 1, frac_pass75 := n_pass75 / n_attmpt]

# merge into invariant dataset
dt.time.invariant <- merge(dt.time.invariant, cross.section.outcomes, by = c("sid"), all.x = TRUE)

dt.time.invariant[is.na(obs.cross.section), obs.cross.section := 0]

##########################################################################################################
## Save

saveRDS(dt.time.varying, file = paste0(cleanDir,"/dt_time_varying.Rds"))
saveRDS(dt.time.invariant, file = paste0(cleanDir,"/dt_time_invariant.Rds"))

rm(list = c("dt.time.varying", "dt.time.invariant"))
gc()