##########################################################################################################
## Program: figuresTables
## Description: formats tables
##########################################################################################################

print("Formatting tables")

library(xtable)
library(stargazer)

# user set path
tabOutDir <- ""

##########################################################################################################
## Global Objects

sample.description <- c("Sample includes all applications for students expected to be in high school following SYEP. Applications are omitted if the student submits multiple applications or
  in ungraded special education following SYEP. Applications to vulnerable youth programs, programs based out of the city, or programs with a greater than 99 percent or less than 0 percent selection rate are omitted.")

covariate.list <- c("female", "black", "asian", "hispanic", "white", "freelu", "redlu", "lep", "esl", "esl.not.lep", "sped", "age_syep_rec", "zread8_rec", "zmath8_rec", 
  "select.prev", "apply.prev", "apply", "worked.prev", "select", "zread8", "zmath8")
covariate.list.mode <- paste0(c("female", "black", "asian", "hispanic", "white", "freelu", "redlu", "lep", "esl", "esl.not.lep", "sped", "age_syep_rec"), ".mode")
covariate.labels <- c("Female", "Black", "Asian", "Hispanic", "White", "Free Lunch", "Red Lunch", "LEP", "ESL", "ESL Not LEP", "Spec Ed", "Age", "Zread", "Zmath", 
  "Select", "Apply", "Apply", "Worked", "Select", "Zread", "Zmath")

covariate.list.inter <- paste0("worked.prev.", covariate.list)
covariate.labels.inter <- paste0("Worked x ", covariate.labels)

outcome.list <- c("ever_attempt","n_attmpt","evr_pass65","n_pass55","n_pass65","n_pass75","ave_score2zyr")
outcome.labels <- c("Any Attempt","N. Attempts","Any Pass 65","N. Pass 55","N. Pass 65","N. Pass 75","ZScore")
outcome.labels.detail <- c("Attempt Any Exam","Number Attempted","Pass Any Exam (65+)", "Number of Exams Passed (55+)","Number of Exams Passed (65+)","Number of Exams Passed (75+)","Avg. Z-Score")

outcome.list.print <- gsub("_", "\\\\_", outcome.list)

##########################################################################################################
## Table 1

tab.1 <- readRDS(paste0(tabDir, "table1.Rds"))

table.1 <- stargazer(tab.1[, .(syepyear, lottery.winners, lottery.losers, total)], covariate.labels = c("Year", "Winners", "Losers", "Total"), 
  title = "Table 1: Sample by Lottery Outcome, 2005 - 2008",
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "15pt")

table.1 <- gsub("caption", "caption*", table.1)

table.1 <- gsub("cccc", "lrrr", table.1)

table.1 <- gsub("NOTES", paste0("\\\\parbox[t]{0.5\\\\textwidth}{\\\\textit{Notes}: ", sample.description, "}"), table.1)

cat(table.1, file = paste0(tabOutDir, "table1.tex"))

##########################################################################################################
## Table 2

## Panel A

tab.2a <- readRDS(paste0(tabDir, "table2a.Rds"))

invisible(lapply(setdiff(colnames(tab.2a), "covariate.list"), function(x) tab.2a[, (x) := as.numeric(get(x))]))

tab.2a[, cov.labels := covariate.labels[match(covariate.list, covariate.list.mode)]]
tab.2a[covariate.list == "zread8", cov.labels := "Z Reading 8th Grade"]
tab.2a[covariate.list == "zmath8", cov.labels := "Z Math 8th Grade"]

table.2a <- stargazer(tab.2a[, .(cov.labels, tab.2.mean.1, tab.2.mean.2, tab.2.frac.diff, tab.2.p.value)], covariate.labels = c("", "First-Time Applicants", "Non-Applicants", "Difference", "P-Value"), 
  title = "Table 2a: Comparison of Applicants and Non-Applicants", digits = 2,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "5pt")

table.2a <- gsub("caption", "caption*", table.2a)

table.2a <- gsub("ccccc", "lrrrr", table.2a)

table.2a <- gsub("NOTES", "\\\\parbox[t]{\\\\textwidth}{\\\\textit{Notes}: First time applicants are defined as the first application made by a student after 2005 for students who did not apply in 2005. 2005 is excluded since we cannot see
  applications made before 2005 and thus we cannot distinguish first-time applicants from repeat applicants in 2005. Non-applicants are defined 
  as students in grades 8-11 and alternative special education who never apply between 2005 and 2008. Limited English Proficiency (LEP)
  is determined by score on the Language Assessment Battery exam. Z Reading and Z Math scores are 8th grade state test scores, standardized 
  by grade and year of administration.}", table.2a)

cat(table.2a, file = paste0(tabOutDir, "table2a.tex"))

## Panel B

tab.2b <- readRDS(paste0(tabDir, "table2b.Rds"))

invisible(lapply(setdiff(colnames(tab.2b), "grade.tab.2"), function(x) tab.2b[, (x) := as.numeric(get(x))]))

table.2b <- stargazer(tab.2b[, .(grade.tab.2, frac.students, n.students)], covariate.labels = c("Grade", "Fraction", "Count"), 
  title = "Table 2b: Grade of First Application", digits = 3,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "15pt")

table.2b <- gsub("caption", "caption*", table.2b)

table.2b <- gsub("ccc", "lrr", table.2b)

table.2b <- gsub("NOTES", "\\\\parbox[t]{0.75\\\\textwidth}{\\\\textit{Notes}: First time applicants are defined as the first application made by a student after 2005 for students who did not apply in 2005. 2005 is excluded since we cannot see
  applications made before 2005 and thus we cannot distinguish first-time applicants from repeat applicants in 2005. Non-applicants are defined 
  as students in grades 8-11 and alternative special education who never apply between 2005 and 2008.}", table.2b)

cat(table.2b, file = paste0(tabOutDir, "table2b.tex"))

##########################################################################################################
## Table 3

tab.3 <- readRDS(paste0(tabDir, "table3.Rds"))

invisible(lapply(setdiff(colnames(tab.3), "var.names"), function(x) tab.3[, (x) := as.numeric(get(x))]))

tab.3[, var.label := outcome.labels.detail[match(var.names, outcome.list)]]

table.3 <- stargazer(tab.3[, .(var.label, count, mean, sd, min, max)], covariate.labels = c("", "N", "Mean", "Std. Dev.", "Min", "Max"), 
  title = "Table 3: Regents Exam Outcomes in School Year Following SYEP (Applications 2005 - 2008)", digits = 2,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "15pt")

table.3 <- gsub("caption", "caption*", table.3)

table.3 <- gsub("cccccc", "lccccc", table.3)

table.3 <- gsub("NOTES", paste0("\\\\parbox[t]{\\\\textwidth}{\\\\textit{Notes}: ", sample.description, "}"), table.3)

cat(table.3, file = paste0(tabOutDir, "table3.tex"))

##########################################################################################################
## Table 4

tab.4 <- readRDS(paste0(tabDir, "table4.Rds"))

table.4 <- stargazer(tab.4[, .(syepyear, frac.worked, number.winners)], covariate.labels = c("", "Fraction of Lottery Winners that Worked", "Number of Winners"), 
  title = "Table 4: SYEP Take-Up Rates (2005 - 2008)", digits = 1,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "15pt")

table.4 <- gsub("caption", "caption*", table.4)

table.4 <- gsub("ccc", "lcc", table.4)

table.4 <- gsub("NOTES", paste0("\\\\parbox[t]{0.8\\\\textwidth}{\\\\textit{Notes}: ", sample.description, "}"), table.4)

cat(table.4, file = paste0(tabOutDir, "table4.tex"))

##########################################################################################################
## Table 5

tab.5 <- readRDS(paste0(tabDir, "table5.Rds"))

table.5 <- stargazer(tab.5, covariate.labels = c("", "2005", "2006", "2007", "2008"), 
  title = "Table 5: Lottery Randomization Results", digits = 2,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "35pt")

table.5 <- gsub("caption", "caption*", table.5)

table.5 <- gsub("NOTES", paste0("\\\\parbox[t]{0.7\\\\textwidth}{\\\\textit{Notes}: We test lottery randomization by regressing each student characteristic on 
  a full set of lottery fixed effects and lottery fixed effects interacted with lottery outcome. We test the restriction that all lottery-by-outcome coefficients are zero. Recall that each year, 
  each CBO conducted its own separate lottery, and therefore we need to test the joint hypothesis that all CBO lottery outcomes are unrelated to student characteristics. Conducting a single test 
  in which we treat all separate lotteries as a single lottery likely biases the test}"), table.5)

cat(table.5, file = paste0(tabOutDir, "table5.tex"))

##########################################################################################################
## Table 6

format.main.reg.table <- function(table, covariate.names, title = NULL, covariate.labels = NULL, col.labels = NULL, notes = NULL, fe.lines = NULL, omit.list = NULL){

  table.format <- copy(table)

  table.format[4] <- gsub("\\\\centering", "\\\\centering\\\\small ", table[4])
  
  coef.row.ind <- seq(15, grep("Observations", table.format) - 4, by = 3)

  if (!is.null(covariate.labels)) {

      for (row.ind in coef.row.ind) {

      cov.name <- substr(table[[row.ind]], 2, regexpr("\\&", table[[row.ind]])[1] - 2)

      cov.label <- covariate.labels[match(gsub("\\\\", "", cov.name), covariate.names)]

      if (!is.na(cov.label)) {

        table.format[[row.ind]] <- gsub(gsub("\\\\", "\\\\\\\\", cov.name), cov.label, table[[row.ind]]) 

      }

    }

  }

  table.format[grep("Note:", table.format)] <- paste0("\\multicolumn{8}{l}{\\parbox[t]{1.1\\textwidth}{\\textit{Note}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. ", notes, "}} \\\\ ")

  table.format[7] <- gsub("5pt", "0pt", table[7])

  table.format[12] <- paste0("\\\\[-1.8ex] & ", paste0(col.labels, collapse = " & "), "\\\\")

  if (!is.null(fe.lines)) {

    fe.rows <- lapply(fe.lines, function(x) paste(paste0(x, collapse = " & "), " \\\\"))

    obs.ind.row <- grep("Observations", table.format)

    table.format <- c(table.format[1:(obs.ind.row - 1)], fe.rows, table.format[obs.ind.row:length(table.format)])

  }

  if (!is.null(omit.list)) {

    coef.row.ind.omit <- grep(paste(gsub("_", "\\\\\\\\_", omit.list), collapse = "|"), table.format)

    full.coef.row.ind.omit <- unlist(lapply(coef.row.ind.omit, function(x) seq(x, x + 2)))

    misc.row.ind.omit <- grep("Table created|Date and time|Adjusted R|Residual Std", table.format)

    table.format <- table.format[setdiff(seq_along(table.format), c(full.coef.row.ind.omit, misc.row.ind.omit))]

  }

  table.format <- gsub("caption\\{\\}", paste0("caption*{", title, "}"), table.format)

  return(table.format)
}

reg.omit.list <- c("miss_gender", "miss_ethn", "missfrl", "age_flag", "zread8_flag", "zmath8_flag")

reg.notes <- "Heteroskedastic robust standard errors clustered at the student-level. Students
in 12th grade, below 8th grade, and in ungraded special education are excluded. Cohort is an indicator for the year of
first application to SYEP interacted with the grade of the student when first applied. There are 24 unique cohorts in
the sample. Limited English Proficiency (LEP) is determined by score on the Language Assessment Battery exam.
Zread and Zmath are 8th grade state test scores, standardized by grade and year of administration. Grade is current
grade level in school which includes 8-11th grade and an additional category for alternative specialized programs (for
example GED programs)"

main.spec.fe.lines <- list(
  c("CBO x Year FE?", "Y", "Y", "Y","Y","Y","Y","Y"), 
  c("Cohort FE?", "Y", "Y", "Y","Y","Y","Y","Y"),
  c("Grade FE?", "Y", "Y", "Y","Y","Y","Y","Y")
  )

table.6 <- readRDS(paste0(tabDir, "table6.Rds"))

table.6.format <- format.main.reg.table(table.6, covariate.list, title = "Table 6: Intent-to-Treat Estimates", covariate.labels = covariate.labels, col.labels = outcome.labels, 
  notes = reg.notes, fe.lines = main.spec.fe.lines, omit.list = reg.omit.list)

cat(table.6.format, file = paste0(tabOutDir, "table6.tex"))

##########################################################################################################
## Table 7

table.7 <- readRDS(paste0(tabDir, "table7.Rds"))

table.7.R2 <- readRDS(paste0(tabDir, "table7_R2.Rds"))

table.7.swap.endog <- c(table.7[1:14], table.7[69:71], table.7[setdiff(seq_along(table.7), c(1:14, 69:71))])

table.7.scrub <- gsub("`|\\(fit\\)", "", table.7.swap.endog)

table.7.scrub[74] <- paste0("R$^{2}$ & ", paste(round(table.7.R2, 3), collapse = "&"), "\\\\")

table.7.format <- format.main.reg.table(table.7.scrub, covariate.list, title = "Table 7: Treatment-on-the-Treated Estimates", covariate.labels = covariate.labels, col.labels = outcome.labels, 
  notes = reg.notes, fe.lines = main.spec.fe.lines, omit.list = reg.omit.list)

cat(table.7.format, file = paste0(tabOutDir, "table7.tex"))

##########################################################################################################
## Table 8

stat.names <- c("LATE", "mean.eb", "p01.eb", "p10.eb", "p50.eb", "p90.eb", "p99.eb", "p50.less.p10.eb", "p90.less.p50.eb")
stat.labels <- c("LATE", "Avg. EB", "P01 EB", "P10 EB", "P50 EB", "P90 EB", "P99 EB", "P50 EB - P10 EB", "P90 EB - P50 EB")

eb.tab.notes <- c("For each outcome, expected benefit (EB) is the predicted treatment 
  effect of SYEP given student covariates and the 2SLS estimates of heterogeneous treatment effects by student covariates. 
  Bootstrap standard errors in parantheses are calculated with 1000 bootstrap iterations, block clustered at the student level. 
  Bootstrap p-values for LATE, mean, and quantiles computed for a two-sided test, and bootstrap p-values for difference in select quantiles computed for a one-sided test.")

build.EB.table <- function(tab.l, title = NULL, notes = NULL){

  tab.l[!like(stat.name, "less"), stat.name.sort := 1]
  tab.l[like(stat.name, "less"), stat.name.sort := 2]

  tab.est <- dcast(tab.l, stat.name + stat.name.sort ~ outcome.name, value.var = "estimate")
  tab.est[, stat.sort := 1]

  tab.se <- dcast(tab.l, stat.name + stat.name.sort ~ outcome.name, value.var = "se")
  tab.se[, stat.sort := 2]

  tab <- rbind(tab.est, tab.se)[order(stat.name.sort, stat.name, stat.sort)]

  invisible(lapply(setdiff(colnames(tab), c("stat.name", "stat.name.sort", "stat.sort")), function(x) tab[, (x) := as.character(round(get(x), 3))]))
  invisible(lapply(setdiff(colnames(tab), c("stat.name", "stat.name.sort", "stat.sort")), function(x) tab[stat.sort == 2, (x) := paste0("(", get(x), ")")]))

  for (stat in unique(tab$stat.name)) {

    for (outcome in outcome.list) {

      if (tab.l[stat.name == stat & outcome.name == outcome, p.val] <= 0.01) {

        tab[stat.name == stat & stat.sort == 1, outcome] <- paste0("DOLLAR", tab[stat.name == stat & stat.sort == 1, get(outcome)], "THREESTARDOLLAR")

      } else if (tab.l[stat.name == stat & outcome.name == outcome, p.val] <= 0.05) {

        tab[stat.name == stat & stat.sort == 1, outcome] <- paste0("DOLLAR", tab[stat.name == stat & stat.sort == 1, get(outcome)], "TWOSTARDOLLAR")

      } else if (tab.l[stat.name == stat & outcome.name == outcome, p.val] <= 0.1) {

        tab[stat.name == stat & stat.sort == 1, outcome] <- paste0("DOLLAR", tab[stat.name == stat & stat.sort == 1, get(outcome)], "ONESTARDOLLAR")

      }

    }

  }

  tab[, stat.label := stat.labels[match(stat.name, stat.names)]]
  tab[stat.sort == 2, stat.label := ""]

  table.formatted <- stargazer(tab[, .(stat.label, ever_attempt, n_attmpt, evr_pass65, n_pass55, n_pass65, n_pass75, ave_score2zyr)], 
    covariate.labels = c("", outcome.labels), 
    title = title, digits = 3,
    notes = "NOTES",
    summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "0pt")

  table.formatted <- gsub("\\\\centering", "\\\\centering\\\\small", table.formatted)

  table.formatted <- gsub("DOLLAR", "$", table.formatted)
  table.formatted <- gsub("THREESTAR", "^{***}", table.formatted)
  table.formatted <- gsub("TWOSTAR", "^{**}", table.formatted)
  table.formatted <- gsub("ONESTAR", "^{*}", table.formatted)

  table.formatted <- gsub("caption", "caption*", table.formatted)
  table.formatted <- gsub("cccccccc", "lccccccc", table.formatted)

  table.formatted <- gsub("NOTES", paste0("\\\\parbox[t]{1.1\\\\textwidth}{\\\\textit{Notes}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. ", notes, "}"), table.formatted)

  return(table.formatted)
}

## Table 8a: Appliers Only

tab.8a.l <- readRDS(paste0(tabDir, "table8a.Rds"))

table.8a <- build.EB.table(tab.8a.l, "Table 8a: LATE and Expected Benefit (Appliers Only)", 
  paste(eb.tab.notes, "Standard errors of LATE estimates differ from those in Table 7 as they are bootstrapped instead of asymptotic estimates."))

cat(table.8a, file = paste0(tabOutDir, "table8a.tex"))

## Table 8b: Non-Appliers Only

tab.8b.l <- readRDS(paste0(tabDir, "table8b.Rds"))

table.8b <- build.EB.table(tab.8b.l, "Table 8b: Expected Benefit (Non-Appliers Only)", 
  gsub("for LATE, mean, and ", "for mean and ", eb.tab.notes))

cat(table.8b, file = paste0(tabOutDir, "table8b.tex"))

##########################################################################################################
## Table 9

tab.9 <- readRDS(paste0(tabDir, "table9.Rds"))

table.9 <- stargazer(tab.9[, .(num.app, frac.app, count, num.win, frac.win, count.win)], covariate.labels = c("N Apps", "Percent", "N Students", "N Wins", "Percent", "N Students"), 
  title = "Table 9: Number of Applications and Selections", digits = 2,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "5pt")

table.9 <- gsub("caption", "caption*", table.9)
table.9 <- gsub("cccccc", "lrrlrr", table.9)

table.9 <- gsub("NOTES", paste0("\\\\parbox[t]{0.7\\\\textwidth}{\\\\textit{Notes}: ", sample.description, "}"), table.9)

cat(table.9, file = paste0(tabOutDir, "table9.tex"))

##########################################################################################################
## Table 10

tab.10a <- readRDS(paste0(tabDir, "table10a.Rds"))
tab.10b <- readRDS(paste0(tabDir, "table10b.Rds"))
tab.10c <- readRDS(paste0(tabDir, "table10c.Rds"))

format.panel <- function(tab){

  panel.format <- c(
  tab[4:17],
  tab[72:81]) 

  panel.format[1] <- gsub("\\\\centering", "\\\\centering\\\\small ", panel.format[1])

  panel.format[12] <- gsub("select.prev", "Select", panel.format[12])

  panel.format[4] <- gsub("5pt", "0pt", panel.format[4])

  panel.format[9] <- paste0("\\\\[-1.8ex] & ", paste0(outcome.labels, collapse = " & "), "\\\\")

  fe.rows <- lapply(main.spec.fe.lines, function(x) paste(paste0(x, collapse = " & "), " \\\\"))

  panel.format <- c(panel.format[1:15], fe.rows, panel.format[16:length(panel.format)])

  misc.row.ind.omit <- grep("Table created|Date and time|Adjusted R|Residual Std", panel.format)

  panel.format <- panel.format[setdiff(seq_along(panel.format), misc.row.ind.omit)]

  return(panel.format)
}

panel.10a <- format.panel(tab.10a)
panel.10b <- format.panel(tab.10b)
panel.10c <- format.panel(tab.10c)

table.10 <- c(
  panel.10a[1:20],
  panel.10b[12:20],
  panel.10c[12:25])

table.10[grep("Note:", table.10)] <- paste0("\\multicolumn{8}{l}{\\parbox[t]{1.1\\textwidth}{\\textit{Note}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. ", 
  "Sample limited to outcomes in 2007-2009.", sample.description, "}} \\\\ ")

table.10[1] <- gsub("\\\\centering", "\\\\centering\\\\small ", table.10[1])

table.10 <- gsub("caption\\{\\}", "caption*{Table 10: Hetereogenous Effects by Past Participation}", table.10)

table.10.format <- c(table.10[1:11], 
  "\\\\[-0.8ex] \\multicolumn{8}{l}{Panel A: First-Time Applicants (Group 1)} \\\\[-0.8ex] \\\\", 
  table.10[12:14],
  "\\cline{2-8} \\\\[-1.8ex]",
  table.10[16:20],
  "\\\\[-1.8ex]\\hline \\\\[-0.8ex]",
  "\\multicolumn{8}{l}{Panel B: Second-Time Applicants, One-Time Past Participators (Group 2)} \\\\[-0.8ex] \\\\", 
  table.10[21:23],
  "\\cline{2-8} \\\\[-1.8ex]",
  table.10[25:29],
  "\\\\[-1.8ex]\\hline \\\\[-0.8ex]",
  "\\multicolumn{8}{l}{Panel C: Third-Time Applicants, Two-Time Past Participators (Group 3)} \\\\[-0.8ex] \\\\", 
  table.10[30:32],
  "\\cline{2-8} \\\\[-1.8ex]",
  table.10[34:43])

cat(table.10.format, file = paste0(tabOutDir, "table10.tex"))

##########################################################################################################
## Table 11

tab.11 <- readRDS(paste0(tabDir, "table11.Rds"))

invisible(lapply(setdiff(colnames(tab.11), "covariate.names"), function(x) tab.11[, (x) := as.numeric(get(x))]))

table.11 <- stargazer(tab.11[, .(covariate.names, group.2.less.1, pval.2.less.1, group.3.less.2, pval.3.less.2)], 
  covariate.labels = c("", "Group 2 Less Group 1", "P-Value", "Group 3 Less Group 2", "P-Value"), 
  title = "Table 11: Balance Test - Hetereogenous Effects Groups", digits = 2,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "5pt")

table.11 <- gsub("caption", "caption*", table.11)
table.11 <- gsub("ccccc", "lrrrr", table.11)
table.11 <- gsub("0\\.0000", "0", table.11)

table.11 <- gsub("NOTES", paste0("\\\\parbox[t]{0.9\\\\textwidth}{\\\\textit{Notes}: Displayed point estimates are differences in average covariate value between Groups 2
and 1 and Groups 3 and 2, respectively. Group 1 consists of all first-time applicants in years
2006-2008. Group 2 is all students who applied for the second time in 2006-2008 and had
applied, won, and participated in the prior year. Group 3 is all students who applied for the
third time in 2006-2008 and had applied, won, and participated in each of the two years prior.}"), table.11)

cat(table.11, file = paste0(tabOutDir, "table11.tex"))
    
##########################################################################################################
## Table 12

tab.12 <- readRDS(paste0(tabDir, "table12.Rds"))

table.12 <- stargazer(tab.12, 
  covariate.labels = c("Group", outcome.labels), 
  title = "Table 12: Average Expected Benefit", digits = 4,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "0pt")

table.12 <- gsub("caption", "caption*", table.12)
table.12 <- gsub("cccccccc", "lrrrrrrr", table.12)

table.12 <- gsub("NOTES", paste0("\\\\parbox[t]{1\\\\textwidth}{\\\\textit{Notes}: For each outcome, expected benefit (EB) is the predicted treatment effect of SYEP given student covariates
and the 2SLS estimates of heterogeneous treatment effects by student covariates. Group 1 consists of all first-time
applicants in years 2006-2008. Group 2 is all students who applied for the second time in 2006-2008 and had applied,
won, and participated in the prior year. Group 3 is all students who applied for the third time in 2006-2008 and had
applied, won, and participated in each of the two years prior.}"), table.12)

cat(table.12, file = paste0(tabOutDir, "table12.tex"))

##########################################################################################################
## Table 13

tab.13 <- readRDS(paste0(tabDir, "table13.Rds"))


table.13 <- stargazer(tab.13, covariate.labels = c("", "Win Coef", "SE", "P-Value"), 
  title = "Table 13: Lottery Randomization Results by Covariate",
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "35pt")

table.13 <- gsub("caption", "caption*", table.13)
table.13 <- gsub("cccc", "lrrr", table.13)

table.13 <- gsub("NOTES", paste0("\\\\parbox[t]{0.7\\\\textwidth}{\\\\textit{Notes}: Point estimates and hetereoskedastic robust standard
errors (clustered at student level) for separate regressions of each student covariate on full set of indicators for winning each lottery
(2005-2008).}"), table.13)

cat(table.13, file = paste0(tabOutDir, "table13.tex"))

##########################################################################################################
## Table 14

tab.14 <- readRDS(paste0(tabDir, "table14.Rds"))

table.14 <- tab.14[4:27] 

table.14[1] <- gsub("\\\\centering", "\\\\centering\\\\small ", table.14[1])

table.14[2] <- "\\caption*{Table 14: Falsification Test (Outcomes Year Prior to Lottery)}"

table.14[12] <- gsub("select", "Select", table.14[12])

table.14[4] <- gsub("5pt", "0pt", table.14[4])

table.14[9] <- paste0("\\\\[-1.8ex] & ", paste0(outcome.labels, collapse = " & "), "\\\\")

fe.rows <- lapply(main.spec.fe.lines, function(x) paste(paste0(x, collapse = " & "), " \\\\"))

table.14 <- c(table.14[1:15], unlist(fe.rows), table.14[16:length(table.14)])

misc.row.ind.omit <- grep("Table created|Date and time|Adjusted R|Residual Std", table.14)

table.14 <- table.14[setdiff(seq_along(table.14), misc.row.ind.omit)]

table.14[grep("Note:", table.14)] <- paste0("\\multicolumn{8}{l}{\\parbox[t]{1.1\\textwidth}{\\textit{Note}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. Sample differs from main analyses due to lagged outcome. ", 
  sample.description, "}} \\\\ ")

cat(table.14, file = paste0(tabOutDir, "table14.tex"))

##########################################################################################################
## Table A1

tab.A1 <- readRDS(paste0(tabDir, "tableA1.Rds"))

tab.A1[, syepyear := as.character(syepyear)]

table.A1 <- stargazer(tab.A1[, .(syepyear, win.prob.matched, win.prob.not.matched, win.prob.total)], 
  covariate.labels = c("", "Matched", "Not Matched", "Total"), 
  title = "Table A1: Likelihood of Winning SYEP Lottery by Matching to DOE Data", digits = 1,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "35pt")

table.A1 <- gsub("caption", "caption*", table.A1)
table.A1 <- gsub("cccc", "lrrr", table.A1)

table.A1 <- gsub("NOTES", paste0("\\\\parbox[t]{0.7\\\\textwidth}{\\\\textit{Notes}: Applications to vulnerable youth programs, pro-
grams based out of the city, or programs with a greater than 99 percent or less than 0 percent selection rate are omitted.}"), table.A1)

cat(table.A1, file = paste0(tabOutDir, "tableA1.tex"))

##########################################################################################################
## Table A2

table.A2 <- readRDS(paste0(tabDir, "tableA2.Rds"))

a2.notes <- "Heteroskedastic robust standard errors clustered at the lottery-
level. Grade is last grade before application which includes 7-12th grade and an additional category
for alternative programs. Sample includes all unmatched applications to SYEP in years 2006-2008
and all first-time applicants in those years. 2005 is excluded since we cannot see applications made
before 2005 and thus we cannot distinguish first-time applicants from repeat applicants in 2005."

a2.fe.lines <- list(
  c("CBO x Year FE?", "Y"), 
  c("Grade FE?", "Y")
  )

table.A2[4] <- gsub("\\\\centering", "\\\\centering\\\\small ", table.A2[4])

table.A2[14] <- gsub("select", "Select", table.A2[14])

table.A2[grep("Note:", table.A2)] <- paste0("\\multicolumn{2}{l}{\\parbox[t]{0.9\\textwidth}{\\textit{Note}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. ", a2.notes, "}} \\\\ ")

table.A2[7] <- gsub("5pt", "0pt", table.A2[7])

table.A2[12] <- paste0("\\\\[-1.8ex] & Matched \\\\")

fe.rows <- lapply(a2.fe.lines, function(x) paste(paste0(x, collapse = " & "), " \\\\"))

table.A2 <- c(table.A2[1:17], fe.rows, table.A2[18:length(table.A2)])

misc.row.ind.omit <- grep("Table created|Date and time|Adjusted R|Residual Std", table.A2)

table.A2 <- table.A2[setdiff(seq_along(table.A2), misc.row.ind.omit)]

table.A2 <- gsub("caption\\{\\}", paste0("caption*{Table A2: Probability of Being Matched to DOE Data (2006 - 2008)}"), table.A2)

cat(table.A2, file = paste0(tabOutDir, "tableA2.tex"))

##########################################################################################################
## Table A3

tab.A3 <- readRDS(paste0(tabDir, "tableA3.Rds"))

table.A3 <- stargazer(tab.A3[, .(grade.label, frac.attrit.win, frac.attrit.lose, frac.attrit.all, n.apps)], 
  covariate.labels = c("Grade", "Frac Attrite Winners", "Frac Attrite Losers", "Frac Attrite All", "N Applications"), 
  title = "Table A3: Attrition in Year Following Application", digits = 1,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "5pt")

table.A3 <- gsub("caption", "caption*", table.A3)
table.A3 <- gsub("ccccc", "lrrrr", table.A3)

table.A3 <- gsub("NOTES", paste0("\\\\parbox[t]{1\\\\textwidth}{\\\\textit{Notes}: Attrition is defined as not appearing in 
  NYCDOE administrative data in the year following the SYEP lottery. Main Analyses condition on non-attrition, 
  so number of analyzed applications is mechanically smaller than total presented here. ", sample.description, "}"), table.A3)

cat(table.A3, file = paste0(tabOutDir, "tableA3.tex"))

##########################################################################################################
## Table A4

tab.A4 <- readRDS(paste0(tabDir, "tableA4.Rds"))

a4.fe.lines <- list(
  c("CBO x Year FE?", "Y", "Y", "Y","Y","Y","Y"), 
  c("Cohort FE?", "Y", "Y", "Y","Y","Y","Y"),
  c("Grade FE?", "N", "N", "N","N","N","Y")
  )

table.A4 <- tab.A4[4:27] 

table.A4[1] <- gsub("\\\\centering", "\\\\centering\\\\small ", table.A4[1])

table.A4[2] <- "\\caption*{Table A4: Impact of Winning Lottery on Attrition (2006 - 2009)}"

table.A4[12] <- gsub("select", "Select", table.A4[12])

table.A4[4] <- gsub("5pt", "0pt", table.A4[4])

table.A4[9] <- paste0("\\\\[-1.8ex] & 8th Grade & 9th Grade & 10th Grade & 11th Grade & Alt Program & All Grades ", "\\\\")

fe.rows <- lapply(a4.fe.lines, function(x) paste(paste0(x, collapse = " & "), " \\\\"))

table.A4 <- c(table.A4[1:15], unlist(fe.rows), table.A4[16:length(table.A4)])

misc.row.ind.omit <- grep("Table created|Date and time|Adjusted R|Residual Std", table.A4)

table.A4 <- table.A4[setdiff(seq_along(table.A4), misc.row.ind.omit)]

table.A4[grep("Note:", table.A4)] <- paste0("\\multicolumn{7}{l}{\\parbox[t]{0.95\\textwidth}{\\textit{Note}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. 
  Outcome variable is attrition in years 2006-2009. Attrition is defined as not appearing in DOE administrative data in the year following the SYEP lottery. 
  Main analyses condition on non-attrition, so number of analyzed applications is mechanically smaller than total presented here. ", 
  sample.description, "}} \\\\ ")

cat(table.A4, file = paste0(tabOutDir, "tableA4.tex"))

##########################################################################################################
## Table A5

tab.A5 <- readRDS(paste0(tabDir, "tableA5.Rds"))

table.A5 <- stargazer(tab.A5[, .(grade.label, frac.ever_attempt)], 
  covariate.labels = c("Grade", "Fraction"), 
  title = "Table A5: Fraction of NYC DOE Students Attempting at Least One Regents Exam 2006 - 2009", digits = 1,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "20pt")

table.A5 <- gsub("caption", "caption*", table.A5)
table.A5 <- gsub("cc", "lr", table.A5)

table.A5 <- gsub("NOTES", paste0("\\\\parbox[t]{0.4\\\\textwidth}{\\\\textit{Notes}: NYC DOE students include all students with non-missing grades who appear in
administrative data.}"), table.A5)

cat(table.A5, file = paste0(tabOutDir, "tableA5.tex"))

##########################################################################################################
## Table A6

tab.A6 <- readRDS(paste0(tabDir, "tableA6.Rds"))

tab.A6[, cov.label := covariate.labels[match(variable, covariate.list)]]
tab.A6[is.na(cov.label), cov.label := paste0("Perc ", covariate.labels[match(gsub("frac\\.", "", variable), covariate.list)])]

table.A6 <- stargazer(tab.A6[, mget(c("cov.label", outcome.list))], 
  covariate.labels = c("", outcome.labels), 
  title = "Table A6: Comparing ITT Effect Size", digits = 3,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "0pt")

table.A6[2] <- gsub("\\\\centering", "\\\\centering\\\\small ", table.A6[2])

table.A6 <- gsub("caption", "caption*", table.A6)
table.A6 <- gsub("cccccccc", "lccccccc", table.A6)

table.A6 <- gsub("NOTES", paste0("\\\\parbox[t]{1.1\\\\textwidth}{\\\\textit{Notes}: For each outcome, the percent of covariate effect 
(Panel B) is defined as the ITT effect of SYEP divided by the absolute value of the coefficient on a given covariate in the estimated ITT model.}"), table.A6)

table.A6 <- c(table.A6[1:7], 
  "\\multicolumn{8}{l}{Panel A: ITT Estimates} \\\\[-0.8ex] \\\\", 
  table.A6[8], 
  "\\\\[-2.8ex]\\cline {2-8} \\\\", 
  "\\\\[-4.8ex]",
  table.A6[10:13],
  "\\\\[-1.8ex]\\hline \\\\[-0.8ex]",
  "\\multicolumn{8}{l}{Panel B: SYEP Effect / Covariate Coefficient} \\\\[-0.8ex] \\\\", 
  table.A6[8], 
  "\\\\[-2.8ex]\\cline {2-8} \\\\", 
  "\\\\[-4.8ex]",
  table.A6[14:16],
  "\\\\[-1.8ex]",
  table.A6[17:20])

table.A6[7] <- gsub("1\\.8", "0\\.8", table.A6[7])

cat(table.A6, file = paste0(tabOutDir, "tableA6.tex"))

##########################################################################################################
## Table A7

tab.A7 <- readRDS(paste0(tabDir, "tableA7.Rds"))

tab.A7[, cov.label := covariate.labels[match(variable, covariate.list)]]
tab.A7[is.na(cov.label), cov.label := paste0("Perc ", covariate.labels[match(gsub("frac\\.", "", variable), covariate.list)])]

table.A7 <- stargazer(tab.A7[, mget(c("cov.label", outcome.list))], 
  covariate.labels = c("", outcome.labels), 
  title = "Table A7: Comparing TOT Effect Size", digits = 3,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "0pt")

table.A7[2] <- gsub("\\\\centering", "\\\\centering\\\\small ", table.A7[2])

table.A7 <- gsub("caption", "caption*", table.A7)
table.A7 <- gsub("cccccccc", "lccccccc", table.A7)

table.A7 <- gsub("NOTES", paste0("\\\\parbox[t]{1.1\\\\textwidth}{\\\\textit{Notes}: For each outcome, the percent of covariate effect 
(Panel B) is defined as the TOT effect of SYEP divided by the absolute value of the coefficient on a given covariate in the estimated TOT model.}"), table.A7)

table.A7 <- c(table.A7[1:7], 
  "\\multicolumn{8}{l}{Panel A: ITT Estimates} \\\\[-0.8ex] \\\\", 
  table.A7[8], 
  "\\\\[-2.8ex]\\cline {2-8} \\\\", 
  "\\\\[-4.8ex]",
  table.A7[10:13],
  "\\\\[-1.8ex]\\hline \\\\[-0.8ex]",
  "\\multicolumn{8}{l}{Panel B: SYEP Effect / Covariate Coefficient} \\\\[-0.8ex] \\\\", 
  table.A7[8], 
  "\\\\[-2.8ex]\\cline {2-8} \\\\", 
  "\\\\[-4.8ex]",
  table.A7[14:16],
  "\\\\[-1.8ex]",
  table.A7[17:20])

table.A7[7] <- gsub("1\\.8", "0\\.8", table.A7[7])

cat(table.A7, file = paste0(tabOutDir, "tableA7.tex"))

##########################################################################################################
## Table A8a

table.A8a <- readRDS(paste0(tabDir, "tableA8a.Rds"))

table.A8a.R2 <- readRDS(paste0(tabDir, "tableA8a_R2.Rds"))

table.A8a.sel.endog <- c(table.A8a[1:14], table.A8a[63:123])

table.A8a.scrub <- gsub("`|\\(fit\\)", "", table.A8a.sel.endog)

table.A8a.scrub[68] <- paste0("R$^{2}$ &", paste(round(table.A8a.R2, 3), collapse = "&"), "\\\\")

table.A8a.format <- format.main.reg.table(table.A8a.scrub, c(covariate.list, covariate.list.inter), title = "Table A8a: Heterogeneous Treatment-on-the-Treated Estimates", 
  covariate.labels = c(covariate.labels, covariate.labels.inter), col.labels = outcome.labels, 
  notes = reg.notes, fe.lines = main.spec.fe.lines, omit.list = reg.omit.list)

cat(table.A8a.format, file = paste0(tabOutDir, "tableA8a.tex"))

##########################################################################################################
## Table A8b

tab.A8b <- readRDS(paste0(tabDir, "tableA8b.Rds"))

table.A8b <- stargazer(tab.A8b[, mget(c("stat.names", outcome.list))], 
  covariate.labels = c("", outcome.labels), 
  title = "Table A8b: Joint Test of Treatment Interactions", digits = 3,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "2pt")

table.A8b <- gsub("caption", "caption*", table.A8b)
table.A8b <- gsub("cccccccc", "lrrrrrrr", table.A8b)

table.A8b <- gsub("NOTES", paste0("\\\\parbox[t]{\\\\textwidth}{\\\\textit{Notes}: F-statistics and p-values from test of joint restriction that all treatment-by-covariate coefficients are zero.}"), table.A8b)

cat(table.A8b, file = paste0(tabOutDir, "tableA8b.tex"))

##########################################################################################################
## Table A9

tab.A9.l <- readRDS(paste0(tabDir, "tableA9.Rds"))

tab.A9.w <- rbind(
  cbind(
    dcast(tab.A9.l, stat.name ~ outcome.name, value.var = c("diff.est.perc.apply")), rep("diff.est", length(unique(tab.A9.l$stat.name)))
    ),
  cbind(
    dcast(tab.A9.l, stat.name ~ outcome.name, value.var = c("p.val")), rep("p.val", length(unique(tab.A9.l$stat.name)))
    )
  )

stat.names <- c("mean.eb", "p01.eb", "p10.eb", "p50.eb", "p90.eb", "p99.eb")
stat.labels <- c("Avg. EB", "P01 EB", "P10 EB", "P50 EB", "P90 EB", "P99 EB")

tab.A9.w[, stat.sort.order := match(stat.name, stat.names)]
tab.A9.w[, stat.label := stat.labels[match(stat.name, stat.names)]]
tab.A9.w[V2 == "p.val", stat.label := ""]

table.A9 <- stargazer(tab.A9.w[order(stat.sort.order, V2), mget(c("stat.label", outcome.list))], 
  covariate.labels = c("", outcome.labels), 
  title = "Table A9: Expected Benefit Moments - Apply Less Non-Apply", digits = 3,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "0pt")

# wrap p-values in parantheses and assign stars
assign.p.val.stars <- function(table){

  table.format <- copy(table)

  p.val.rows <- grep("^ & \\$", table.format)

  for (row.ind in p.val.rows) {

    new.p.val.row <- list()
    new.est.row <- list()

    p.val.row <- strsplit(table.format[row.ind], "&")[[1]]
    est.val.row <- strsplit(table.format[row.ind - 1], "&")[[1]]

    est.name <- est.val.row[[1]]

    for (outcome.i in seq_along(p.val.row[2:length(p.val.row)]) ) {

      p.val.num <- as.numeric(gsub("\\$|\\\\", "", p.val.row[outcome.i + 1]))
      est.val.num <- as.numeric(gsub("\\$|\\\\", "", est.val.row[outcome.i + 1]))

      replace.p.val <- paste0(" $(", p.val.num, ")$")

      if ( p.val.num <= 0.01) {

        replace.est.val <- paste0(" $", est.val.num, "^{***}$")

      } else if ( p.val.num <= 0.05 ) {

        replace.est.val <- paste0(" $", est.val.num, "^{**}$")

      } else if ( p.val.num <= 0.1 ) {

        replace.est.val <- paste0(" $", est.val.num, "^{*}$")

      } else {

        replace.est.val <- paste0(" $", est.val.num, "$")

      }

      new.p.val.row[[length(new.p.val.row) + 1]] <- replace.p.val
      new.est.row[[length(new.est.row) + 1]] <- replace.est.val

    }

    table.format[row.ind] <- paste(" &", paste(unlist(new.p.val.row), collapse = "&"), " \\\\")
    table.format[row.ind - 1] <- paste(est.name, " &", paste(unlist(new.est.row), collapse = "&"), " \\\\")

  }

  return(table.format)
}

table.A9.stars <- assign.p.val.stars(table.A9)
    
table.A9.stars <- gsub("caption", "caption*", table.A9.stars)
table.A9.stars <- gsub("cccccccc", "lccccccc", table.A9.stars)

table.A9.stars <- gsub("NOTES", paste0("\\\\parbox[t]{1.1\\\\textwidth}{\\\\textit{Notes}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. Estimate presented is moment computed on appliers less that computed on non-appliers, expressed as a percentage of applier moment. 
  Bootstrap p-values in parentheses computed against the null hypothesis of no difference between moment for appliers and non-appliers. ", eb.tab.notes, "}"), table.A9.stars)

cat(table.A9.stars, file = paste0(tabOutDir, "tableA9.tex"))

##########################################################################################################
## Tables A10a-b

tab.A10a <- readRDS(paste0(tabDir, "tableA10a.Rds"))

tab.A10b <- readRDS(paste0(tabDir, "tableA10b.Rds"))

gen.tab.A10 <- function(tab.A10, tab.notes = NULL, title = "TITLE"){

  tab.A10.w <- rbind(
  cbind(
    dcast(tab.A10, cov.name ~ outcome.name, value.var = c("diff")), rep("diff", length(unique(tab.A10$cov.name)))
    ),
  cbind(
    dcast(tab.A10, cov.name ~ outcome.name, value.var = c("p.val")), rep("p.val", length(unique(tab.A10$cov.name)))
    )
  )

  tab.A10.w[, cov.label := covariate.labels[match(cov.name, covariate.list)]]
  tab.A10.w[V2 == "p.val", cov.label := ""]

  table.A10 <- stargazer(tab.A10.w[order(cov.name, V2), mget(c("cov.label", outcome.list))], 
    covariate.labels = c("", outcome.labels), 
    title = title, digits = 3,
    notes = "NOTES",
    summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "0pt")

  table.A10.stars <- assign.p.val.stars(table.A10)

  table.A10.stars <- gsub("caption", "caption*", table.A10.stars)
  table.A10.stars <- gsub("cccccccc", "lccccccc", table.A10.stars)

  table.A10.stars <- gsub("NOTES", paste0("\\\\parbox[t]{1.1\\\\textwidth}{\\\\textit{Notes}: $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01. ", tab.notes, "}"), table.A10.stars)

  return(table.A10.stars)
}

notes.A10 <- c("Displayed are differences in mean covariates between between top and bottom deciles of expected benefit: 
  E[X | EB quantile=90, outcome=Y]-E[X | EB quantile=10, outcome=Y]. P-values for t-test of equality of means displayed.")

table.A10a <- gen.tab.A10(tab.A10a, tab.notes = paste(notes.A10, "Appliers only."), title = "Table A10a: 90th-10th EB Quantile Differential in Mean Covariates (Appliers)")

cat(table.A10a, file = paste0(tabOutDir, "tableA10a.tex"))

table.A10b <- gen.tab.A10(tab.A10b, tab.notes = paste(notes.A10, "Non-appliers only."), title = "Table A10b: 90th-10th EB Quantile Differential in Mean Covariates (Non-Appliers)")

cat(table.A10b, file = paste0(tabOutDir, "tableA10b.tex"))

##########################################################################################################
## Table A11

tab.A11 <- readRDS(paste0(tabDir, "tableA11.Rds"))

table.A11 <- stargazer(tab.A11[, mget(c("stat.name", "pooled", "group.1", "group.2", "group.3"))], 
  covariate.labels = c("", "Full Sample", "Group 1", "Group 2", "Group 3"), 
  title = "Table A11: Joint Test of Zero Treatment Effect on Outcomes 1-6", digits = 3,
  notes = "NOTES",
  summary = FALSE, header = FALSE, rownames = FALSE, type = "latex", column.sep.width = "20pt")

table.A11 <- gsub("caption", "caption*", table.A11)
table.A11 <- gsub("ccccc", "lrrrr", table.A11)

table.A11 <- gsub("NOTES", paste0("\\\\parbox[t]{0.75\\\\textwidth}{\\\\textit{Notes}: For each group, statistics generated from F-test of joint restriction that treatment
effect is zero for all outcomes other than average Z-score. Average Z-score is omitted
as students who attempt no exams have no defined average score. Group 1 consists
of all first-time applicants in years 2006-2008. Group 2 is all students who applied for
the second time in 2006-2008 and had applied, won, and participated in the prior year.
Group 3 is all students who applied for the third time in 2006-2008 and had applied,
won, and participated in each of the two years prior. Full Sample is identical to that
analyzed in Tables 6-7.}"), table.A11)

cat(table.A11, file = paste0(tabOutDir, "tableA11.tex"))
