##########################################################################################################
## Program: utilities
## Description: holds utilities
##########################################################################################################

library(data.table)
library(lfe)
library(parallel)
library(stargazer)
library(haven)
library(lubridate)

##########################################################################################################
## Misc

# create interactions between two variables
create.inter <- function(dt, var.1, var.2){
  
  if (length(grep(paste0(var.1, ".", var.2), colnames(dt))) == 0){
    dt[, temp_var.1 := dt[, var.1, with = FALSE]]
    dt[, temp_var.2 := dt[, var.2, with = FALSE]]
    
    dt[, (paste0(var.1, ".", var.2)) := temp_var.1*temp_var.2]
    
    dt[, temp_var.1 := NULL]
    dt[, temp_var.2 := NULL]
  }
  
}

# replicate Stata's "maxmode" function that finds mode of a variable and breaks ties with maximum
max.mode <- function(x){
  ux <- sort(na.omit(x), decreasing = TRUE)
  ux[which.max(tabulate(match(x, ux)))]
}

# max returning NA instead of -Inf if all values are NA
max2 <- function(x){
  as.numeric(ifelse(!all(is.na(x)), max(x, na.rm = TRUE), NA))
}

##########################################################################################################
## Regressions and Statistics

## OLS Function

reg.OLS <- function(dt, outcome.var, regressors, fixed.effects = "0", inter.list.1 = c(), inter.list.2 = c(), cluster.group = c()){
  
  # create interactions between lists 1 and 2
  if ((length(inter.list.1) > 0) & (length(inter.list.2) > 0)){
    interactions <- paste(rep(inter.list.1, each = length(inter.list.2)), inter.list.2, sep = ":")
  } else {
    interactions <- c()
  }
  
  if (length(cluster.group) > 0){
    model <- felm(as.formula(paste(outcome.var,"~",paste(c(regressors, interactions), collapse = "+")," | ", paste(fixed.effects, collapse = "+"), 
      "| 0 |", paste(cluster.group, collapse = "+"))), data = dt)
  } else {
    model <- felm(as.formula(paste(outcome.var,"~",paste(c(regressors, interactions), collapse = "+")," | ", paste(fixed.effects, collapse = "+"))), data = dt)
  }
  
  return(model)
}

## 2SLS Function

reg.2SLS <- function(dt, outcome.var, treatment.var, instrument.var, regressors, fixed.effects, 
  interact.with.treatment = c(), inter.list.1 = c(), inter.list.2 = c(), cluster.group = c()){
  
  # create interactions between lists 1 and 2
  if ((length(inter.list.1) > 0) & (length(inter.list.2) > 0)){
    interactions <- paste(rep(inter.list.1, each = length(inter.list.2)), inter.list.2, sep = ".")
  } else {
    interactions <- c()
  }
  
  # create interactions with treatment variable and instrument
  if ((length(interact.with.treatment) > 0)) {
    treatment.interactions <- paste(rep(treatment.var, each = length(interact.with.treatment)), interact.with.treatment, sep = ".")
    instrument.interactions <- paste(rep(instrument.var, each = length(interact.with.treatment)), interact.with.treatment, sep = ".")
  } else {
    treatment.interactions <- c()
    instrument.interactions <- c()
  }
  
  endog.vars <- c(treatment.var, treatment.interactions)
  instrument.vars <- c(instrument.var, instrument.interactions)
  
  if (length(cluster.group)>0){
    model.tot <- felm(as.formula(paste(outcome.var, "~", paste(c(regressors, interactions), collapse = "+"),
      " | ", paste(fixed.effects, collapse = "+"), "|",
      "(", paste(endog.vars, collapse = "|"), "~", paste(instrument.vars, collapse = "+"), ")", 
      "|", paste(cluster.group, collapse = "+"))), data = dt)
  } else {
    model.tot <- felm(as.formula(paste(outcome.var, "~", paste(c(regressors, interactions), collapse = "+"),
      " | ", paste(fixed.effects, collapse = "+"), "|",
      "(", paste(endog.vars, collapse = "|"), "~", paste(instrument.vars, collapse = "+"), ")")), data = dt)
  }
  
  return(model.tot)
}

# comparing characteristics of groups, computing mean value of covariate and testing if difference is significant
test.mean.diffs <- function(dt.1, dt.2, covariate, covariate.miss.flag = "0"){
  
  if (covariate.miss.flag != "0"){
    dt.1[, ignore.flag := as.numeric(get(covariate.miss.flag) == 1)]
    dt.2[, ignore.flag := as.numeric(get(covariate.miss.flag) == 1)]
  } else {
    dt.1[, ignore.flag := 0]
    dt.2[, ignore.flag := 0]
  }
  
  mean.1 <- dt.1[ignore.flag == 0, .(frac = mean(get(covariate), na.rm = TRUE))]
  
  mean.2 <- dt.2[ignore.flag == 0, .(frac = mean(get(covariate), na.rm = TRUE))]
  
  # record difference and test if significant
  frac.diff <- mean.1 - mean.2
  
  p.val <- t.test(dt.1[ignore.flag == 0, covariate, with = FALSE], dt.2[ignore.flag == 0, covariate, with = FALSE])[["p.value"]]
  
  return(c(mean.1, mean.2, frac.diff, p.val))
}

##########################################################################################################
## Expected Benefit - Prediction

## Compute Marginal Effects

# for given outcome, run 2SLS on selected regressors used for prediction and covariates not for prediction and store coefficients
reg.2SLS.marginal.effects <- function(dt, outcome.var, treatment.var, instrument.var, regressors, fixed.effects, cluster.group = c("year"), predict.coefs = NULL){
    
  # allow for regressors interacted with treatment to differ from regressors in prediction regression
  if (is.null(predict.coefs)) {
    
    predict.coefs <- regressors
  
  }
  
  # run 2SLS interacting treatment with all regressors
  model.tot <- reg.2SLS(dt, outcome.var, treatment.var, instrument.var, regressors, fixed.effects, interact.with.treatment = predict.coefs, cluster.group = cluster.group)
  
  # extract coefficients for endogenous variables
  endog.vars <- c(treatment.var, vapply(predict.coefs, function(x)(paste0(treatment.var, ".", x)), character(1)))
  
  tot.coef <- coef(model.tot)[grep(paste(endog.vars, collapse = "|"), names(coef(model.tot)))]
  
  return(tot.coef)
}

# for each student in given dt, compute expected marginal benefit given coefficients from 2SLS
predict.marginal.benefit <- function(dt.predict, outcome.var, treatment.var, iv.coefs, eb.var.prefix = "eb."){
  
  # retain only non-NA coefficients
  iv.coefs.valid <- iv.coefs[!is.na(iv.coefs)]
  
  # for each interaction, extract interaction variable name
  regressors.predict <- vapply(names(iv.coefs.valid)[2:length(names(iv.coefs.valid))], function(x) substr(x, nchar(paste0(treatment.var)) + 3, nchar(x) - 6), character(1))
  
  # rename iv coefficients by interaction variable only
  names(iv.coefs.valid) <- c("beta1", regressors.predict)
  
  # calculate expected marginal effect
  marginal.effect <- Reduce("+", lapply(regressors.predict, function(x) dt.predict[[x]] * iv.coefs.valid[[x]])) + iv.coefs.valid["beta1"]
  
  # attach to dt
  dt.predict[, paste0(eb.var.prefix, outcome.var) := marginal.effect]
  
  return(dt.predict)
}

# function that estimates coefficients and appends marginal benefit to prediction dt
estimate.predict.marginal.benefit <- function(dt.est, dt.predict, outcome.var, treatment.var, instrument.var, regressors, fixed.effects, 
  cluster.group = c("year"), predict.coefs = NULL, eb.var.prefix = "eb."){
  
  # get coefficients
  iv.coefs.est <- reg.2SLS.marginal.effects(dt.est, outcome.var, treatment.var, instrument.var, regressors, fixed.effects, 
    predict.coefs = predict.coefs, cluster.group = cluster.group)
  
  # compute expected marginal benefit and append to given dt
  predict.marginal.benefit(dt.predict, outcome.var, treatment.var, iv.coefs.est, eb.var.prefix = eb.var.prefix)
}

##########################################################################################################
## Expected Benefit - Bootstrap SE

## LATE

bootstrap.LATE.SE <- function(dt, outcome.var.list, B, treatment.var, instrument.var, regressors, fixed.effects, cluster.group = c("year"), block.cluster = FALSE, 
  run.parallel = FALSE, n.cores = 10){
  
  seed.list <- 

  # sample and estimate LATE, repeating B times
  if (run.parallel) {

    LATE.list <- mclapply(1:B, sample.est.LATE, dt = dt, outcome.var.list = outcome.var.list, treatment.var = treatment.var, 
     instrument.var = instrument.var, regressors = regressors, fixed.effects = fixed.effects, cluster.group = cluster.group, block.cluster = block.cluster, 
     mc.cores = n.cores)

  } else {

    LATE.list <- lapply(1:B, sample.est.LATE, dt = dt, outcome.var.list = outcome.var.list, treatment.var = treatment.var, 
     instrument.var = instrument.var, regressors = regressors, fixed.effects = fixed.effects, cluster.group = cluster.group, block.cluster = block.cluster)

  }
    
  # create data table with estimates for each outcome
  dt.LATE <- as.data.table(lapply(1:length(outcome.var.list), function(outcome.i) unlist(lapply(LATE.list, function(y) y[[outcome.i]]))))
  colnames(dt.LATE) <- paste0("LATE.", outcome.var.list)
  
  return(dt.LATE)
}

sample.est.LATE <- function(b.iter, dt, outcome.var.list, treatment.var, instrument.var, regressors, fixed.effects, cluster.group = c("year"), block.cluster = FALSE){
  
  # draw sample
  if (block.cluster) {

    sample.block <- as.data.table(dt[, unique(sid)])[sample(.N, .N, replace = TRUE)]
    sample.dt <- dt[sid %in% sample.block$V1]
  
  } else {
  
    sample.dt <- dt[sample(.N, .N, replace = TRUE)]
  
  }
  
  print(paste0("LATE Iter: ", b.iter, " Block Cluster: ", block.cluster, " N Rows: ", nrow(sample.dt)))
      
  # compute TOT estimates
  tot.2sls.b <- lapply(outcome.var.list, reg.2SLS, dt = sample.dt, treatment.var = treatment.var, instrument.var = instrument.var, 
    regressors = regressors, fixed.effects = fixed.effects, cluster.group = cluster.group)
  
  # extract LATE estimates
  LATE.est.list <- lapply(1:length(outcome.var.list), function(x) coef(tot.2sls.b[[x]])["`worked.prev(fit)`"])
 
  return(LATE.est.list)  
}

## Expected Benefit Statistics

bootstrap.EB.stat.SE <- function(dt.est, dt.predict, outcome.var.list, B, treatment.var, instrument.var, regressors, 
  fixed.effects.predict, cluster.group = c("year"), predict.coefs = NULL, 
  by.var.list = list(c("female")), by.var.list.labels = list(c("sex")), block.cluster = FALSE, apply.only = FALSE, 
  run.parallel = FALSE, n.cores = 10){

  # sample and estimate EB statistics for non-appliers and appliers, repeating B times
  if (run.parallel) {

    dt.EB.stat.list <- mclapply(1:B, sample.est.EB, dt.est = dt.est, dt.predict = dt.predict, outcome.var.list = outcome.var.list, treatment.var = treatment.var, 
      instrument.var = instrument.var, regressors = regressors, fixed.effects.predict = fixed.effects.predict,
      predict.coefs = predict.coefs, cluster.group = cluster.group, by.var.list = by.var.list, by.var.list.labels = by.var.list.labels, 
      block.cluster = block.cluster, apply.only = apply.only, 
      mc.cores = n.cores)

  } else {

    dt.EB.stat.list <- lapply(1:B, sample.est.EB, dt.est = dt.est, dt.predict = dt.predict, outcome.var.list = outcome.var.list, treatment.var = treatment.var, 
      instrument.var = instrument.var, regressors = regressors, fixed.effects.predict = fixed.effects.predict,
      predict.coefs = predict.coefs, cluster.group = cluster.group, by.var.list = by.var.list, by.var.list.labels = by.var.list.labels, 
      block.cluster = block.cluster, apply.only = apply.only)

  }
  
  
  # combine data tables for each iteration
  dt.EB.stat.apply <- rbindlist(lapply(dt.EB.stat.list, function(x) x[[1]]))
  dt.EB.stat.nonapply <- rbindlist(lapply(dt.EB.stat.list, function(x) x[[2]]))
  
  return(list(dt.EB.stat.apply, dt.EB.stat.nonapply))
}

sample.est.EB <- function(b.iter, dt.est, dt.predict, outcome.var.list, treatment.var, instrument.var, regressors,
  fixed.effects.predict, cluster.group = c("year"), predict.coefs = NULL,
  by.var.list = list(c("female")), by.var.list.labels = list(c("sex")), block.cluster = FALSE, apply.only = FALSE){

  # draw sample from estimating and prediction datasets
  if (block.cluster) {

    sample.block.est <- as.data.table(dt.est[, unique(sid)])[sample(.N, .N, replace = TRUE)]
    sample.block.predict <- as.data.table(dt.predict[, unique(sid)])[sample(.N, .N, replace = TRUE)]
    
    sample.dt.est <- dt.est[sid %in% sample.block.est$V1]
    sample.dt.predict <- dt.predict[sid %in% sample.block.predict$V1, colnames(dt.predict)[!grepl("^eb.", colnames(dt.predict))], with = FALSE]
  
  } else {
  
    sample.dt.est <- dt.est[sample(.N, .N, replace = TRUE)]
    sample.dt.predict <- dt.predict[sample(.N, .N, replace = TRUE), colnames(dt.predict)[!grepl("^eb.", colnames(dt.predict))], with = FALSE]
  
  }
  
  print(paste0("EB Stat. Iter: ", b.iter, " Block Cluster: ", block.cluster, " Estimation N Rows: ", nrow(sample.dt.est), " Prediction N Rows: ", nrow(sample.dt.predict), " Apply Only: ", apply.only))
  
  setDT(sample.dt.predict)

  # compute EB for each student and outcome
  invisible(lapply(outcome.var.list, estimate.predict.marginal.benefit,
    dt.est = sample.dt.est, dt.predict = sample.dt.predict,
    treatment.var = treatment.var, instrument.var = instrument.var,
    regressors = regressors, fixed.effects = fixed.effects.predict, predict.coefs = predict.coefs, cluster.group = cluster.group))

  eb.var.list <- paste0("eb.",outcome.var.list)
  
  #= EB Statistics =#
  
  ## Appliers
  
  # compute EB statistics for pooled sample
  dt.EB.apply.pooled <- compute.EB.stats(sample.dt.predict[apply == 1 & app.drop.flag == 0], eb.var.list, NULL)
  
  dt.EB.apply.pooled[, by.var.label := "pooled"]
  
  # compute EB statistics for each cell in list of by variables
  if (length(by.var.list) == 0) {
    
    dt.EB.apply.cell.list <- NULL
  
  } else {
    
    dt.EB.apply.cell.list <- lapply(by.var.list, compute.EB.stats, dt = sample.dt.predict[apply == 1 & app.drop.flag == 0], eb.var.list = eb.var.list)
    
    for (label.i in 1:length(by.var.list.labels)) {

      dt.EB.apply.cell.list[[label.i]][, by.var.label := by.var.list.labels[[label.i]]]
    
    }
    
  }
  
  # append pooled
  dt.EB.apply.cell.list[[length(by.var.list) + 1]] <- dt.EB.apply.pooled
  dt.EB.apply.stack <- rbindlist(dt.EB.apply.cell.list, fill = TRUE)
  
  # flag with iteration index
  dt.EB.apply.stack[, iter := b.iter]
  
  ## Non-Appliers
  
  if (!apply.only) {
    
    # compute EB statistics for pooled sample
    dt.EB.nonapply.pooled <- compute.EB.stats(sample.dt.predict[apply == 0], eb.var.list, NULL)
    
    dt.EB.nonapply.pooled[, by.var.label := "pooled"]
    
    # compute EB statistics for each cell in list of by variables
    if (length(by.var.list) == 0) {
      
      dt.EB.nonapply.cell.list <- NULL
    
    } else {
      
      dt.EB.nonapply.cell.list <- lapply(by.var.list, compute.EB.stats, dt = sample.dt.predict[apply == 0], eb.var.list = eb.var.list)
      
      for (label.i in 1:length(by.var.list.labels)) {
      
        dt.EB.nonapply.cell.list[[label.i]][, by.var.label := by.var.list.labels[[label.i]]]
      
      }
      
    }
    
    # append pooled
    dt.EB.nonapply.cell.list[[length(by.var.list) + 1]] <- dt.EB.nonapply.pooled
    dt.EB.nonapply.stack <- rbindlist(dt.EB.nonapply.cell.list, fill = TRUE)
    
    # flag with iteration index
    dt.EB.nonapply.stack[, iter := b.iter]
    
  } else {
    
    dt.EB.nonapply.stack <- NULL
    
  }
  
  return(list(dt.EB.apply.stack, dt.EB.nonapply.stack))
}

compute.EB.stats <- function(dt, eb.var.list, by.vars){

  dt[, 
  c(
    cell.weight = .N / nrow(dt),
    mean = setNames(lapply(eb.var.list, function(x) mean(get(x), na.rm = TRUE)), eb.var.list),
    p01 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.01, na.rm = TRUE)), eb.var.list),
    p10 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.1, na.rm = TRUE)), eb.var.list),
    p25 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.25, na.rm = TRUE)), eb.var.list),
    p50 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.5, na.rm = TRUE)), eb.var.list),
    p75 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.75, na.rm = TRUE)), eb.var.list),
    p90 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.9, na.rm = TRUE)), eb.var.list),
    p99 = setNames(lapply(eb.var.list, function(x) quantile(get(x), probs = 0.99, na.rm = TRUE)), eb.var.list)
  ), by = by.vars]

}

gen.bootstrap.se.pvals <- function(dt.bootstrap, eb.stats.est, stat.names){

  ## Estimates

  # melt estimates
  eb.stats.est[, moment.name := "estimate"]

  eb.est.l <- melt(eb.stats.est, id.vars = "moment.name", measure.vars = grep("^mean|^p01|^p10|^p50|^p90|^p99|^LATE", colnames(eb.stats.est), value = TRUE))
  eb.est.l[, variable := as.character(variable)]
  eb.est.l[, var.name := variable]

  extract.outcome.stat(eb.est.l)

  eb.est <- dcast(eb.est.l, outcome.name + stat.name ~ moment.name, value.vars = c("value"))

  ## Standard Errors

  eb.stats.se <- dt.bootstrap[, c(se = setNames(lapply(stat.names, function(x) var(get(x), na.rm = TRUE)^0.5), stat.names))]

  eb.stats.se.l <- melt(eb.stats.se, id.vars = NULL, measure.vars = colnames(eb.stats.se))
  eb.stats.se.l[, variable := as.character(variable)]
  eb.stats.se.l[substr(variable, 1, 2) == "se", c("moment.name", "var.name") := list("se", substr(variable, 4, nchar(variable)))]

  extract.outcome.stat(eb.stats.se.l)
  
  eb.se <- dcast(eb.stats.se.l, outcome.name + stat.name ~ moment.name, value.vars = c("value"))

  eb.est.se <- merge(eb.est, eb.se, by = c("stat.name", "outcome.name"))

  ## P-Values

  # melt bootstrap iterations by statistic type and iteration
  dt.bootstrap.l <- melt(dt.bootstrap, id.vars = c("iter"), measure.vars = grep("^p|^mean|^LATE", colnames(dt.bootstrap), value = TRUE), variable.factor = FALSE)
  dt.bootstrap.l[, var.name := as.character(variable)]
  dt.bootstrap.l[, estimate.i := value]
  
  extract.outcome.stat(dt.bootstrap.l)

  # merge in standard errors and estimates
  dt.bootstrap.l[, variable := NULL]
  dt.bootstrap.l[eb.est.se, on = c("stat.name", "outcome.name"), c("estimate", "se") := list(estimate, se)]
  
  #= Two-Tailed Test =#
  
  # compute bootstrap t-stats
  dt.bootstrap.l[!grep("less", stat.name), abs.t.stat.i := abs(estimate.i - estimate)/se]
  
  # compute t-stat under H0=0 (of true distribution, replicated on each iteration for convenience)
  dt.bootstrap.l[!grep("less", stat.name), abs.t.stat.H0 := abs(estimate - 0)/se]
  
  # p-val = fraction of distribution that t-stat under H0=0 is above (abs value)
  p.vals.l.two <- dt.bootstrap.l[!grep("less", stat.name), .(p.val = mean(as.numeric(abs.t.stat.H0 <= abs.t.stat.i))), by =. (outcome.name, stat.name)]

  #= One-Tailed Test =#
  
  # compute bootstrap t-stats
  dt.bootstrap.l[grep("less", stat.name), t.stat.i := (estimate.i - estimate)/se]
  
  # compute t-stat under H0=0 (of true distribution, replicated on each iteration for convenience)
  dt.bootstrap.l[grep("less", stat.name), t.stat.H0 := (estimate - 0)/se]
  
  # p-val = fraction of distribution that t-stat under H0=0 is above (abs value)
  p.vals.l.one <- dt.bootstrap.l[grep("less", stat.name), .(p.val = mean(as.numeric(t.stat.H0 <= t.stat.i))), by = .(outcome.name, stat.name)]
  
  #= Combine =#
  
  p.vals.out <- rbind(p.vals.l.two, p.vals.l.one)

  all.out <- merge(eb.est.se, p.vals.out, by = c("stat.name", "outcome.name"))

  return(all.out)
}

extract.outcome.stat <- function(dt.l){

  setDT(dt.l)

  dt.l[substr(var.name,1,4) == "mean", c("stat.name", "outcome.name") := list("mean.eb", substr(var.name, 9, nchar(var.name)))]
  dt.l[substr(var.name,1,4) != "mean", c("stat.name", "outcome.name") := list(substr(var.name, 1, 6), substr(var.name, 8, nchar(var.name)))]
  dt.l[substr(var.name,1,4) == "LATE", c("stat.name", "outcome.name") := list("LATE", substr(var.name, 6, nchar(var.name)))]
  dt.l[grep("less",var.name), c("stat.name","outcome.name") := list(substr(var.name, 1, 15), substr(var.name, 17, nchar(var.name)))]

}
